from __future__ import annotations

import datetime as dt

from .common import DeclarationAccordClient
from .const import TypeDonneesV1, TypeSite


class Contrat(dict):

    def __init__(self, acteur_marche_code=None, contrat_type=None):
        _kwargs = {'contratId': ""}
        if acteur_marche_code is not None:
            _kwargs["acteurMarcheCode"] = acteur_marche_code
        if contrat_type is not None:
            _kwargs['contratType'] = contrat_type
        dict.__init__(self, **_kwargs)


class AccesDonnees(dict):

    def __init__(self, date_debut: dt.date, date_fin: dt.date, type_donnees: TypeDonneesV1, type_site: TypeSite,
                 declaration_accord_client: DeclarationAccordClient):
        _kwargs = {'dateDebut': date_debut, 'dateFin': date_fin, 'typeDonnees': type_donnees.value,
                   'declarationAccordClient': declaration_accord_client}
        if type_site == TypeSite.SOUTIRAGE:
            _kwargs['soutirage'] = True
        if type_site == TypeSite.INJECTION:
            _kwargs['injection'] = True
        dict.__init__(self, **_kwargs)


class DonneesGeneralesAccesDonnees(dict):
    def __init__(self, point_id: str, contrat: Contrat, ref_externe=None):
        _kwargs = {'objetCode': "AME", 'pointId': point_id, 'initiateurLogin': "", 'contrat': contrat}
        if ref_externe is not None:
            _kwargs['refExterne'] = ref_externe
        dict.__init__(self, **_kwargs)


class DemandeAccesDonneesMesures(dict):

    def __init__(self, donnees_generales: DonneesGeneralesAccesDonnees, acces_donnees: AccesDonnees):
        _kwargs = {'donneesGenerales': donnees_generales, 'accesDonnees': acces_donnees}
        dict.__init__(self, **_kwargs)



from django.db import models

class TraceActivite(models.Model):
    demandeur = models.CharField(max_length=50)
    RAE_concerne = models.CharField(max_length=20)
    service_concerne = models.CharField(max_length=100)
    horodate = models.DateTimeField(auto_now_add=True)


import logging
from pprint import pprint

from zeep.helpers import serialize_object

from enedis.connection.enedis_connection import EnedisConnection
from enedis.data_models.recherchepoint import RechercherPoint

logger = logging.getLogger(__name__)


class RecherchePoint:
    ns = 'http://www.enedis.fr/sge/b2b/services/rechercherpoint/v2.0'
    tn_binding = 'http://www.enedis.fr/sge/b2b/services'
    name = 'RecherchePoint-v2.0'
    service_name = 'rechercherPoint'
    port_binding = 'RecherchePointBinding'
    service_path = 'RecherchePoint/v2.0'

    @classmethod
    def recherche_point(cls, connection: EnedisConnection, donnees: RechercherPoint, send=True):
        message, response = None, None
        try:
            service, factory, client = connection.soap_service(cls.name, cls.ns, cls.port_binding, cls.service_path,
                                                           cls.tn_binding)
            donnees['loginUtilisateur'] = connection.login
            message = factory.RechercherPointType(**donnees)

            if send:
                response = serialize_object(service.rechercherPoint(**{k:message[k] for k,v in donnees.items()}), dict)

            return serialize_object(message, dict), response

        except Exception as e:
            logger.exception("Erreur appel service")
            return message, {'erreur': str(e.args), 'status': 'failed'}




from django.test import TestCase
from rest_framework.test import APIClient

from enedis.data_models.commandeaccesdonneesmesures import *
from enedis.data_models.common import *


class TestViewCommandeAccesServicesSouscrit(TestCase):
    maxDiff = None

    def test_view_commanderaccesservicessouscrits(self):
        client = APIClient()
        personne_morale = PersonneMorale('denominationsociale')
        declaration_accord_client = DeclarationAccordClient(personne_morale, accord=True)
        acces_donnees = AccesDonnees(dt.date(2020, 6, 22), dt.date(2022, 6, 22), TypeDonneesV1.CDC,
                                     TypeSite.SOUTIRAGE,
                                     declaration_accord_client)
        donnees_generales = DonneesGeneralesAccesDonnees('12345678911234', Contrat())
        donnees = DemandeAccesDonneesMesures(donnees_generales, acces_donnees)
        entrees = {'serveur': 'poste_travail', 'donnees': donnees}
        reponse = client.post('/ws/v1.0/commandeaccesdonneesmesures/', entrees, format='json')

        resultat_attendu = {'header':
            {
                'acquittement': None},
            'body': {
                'affaireId': 'string',
                'prestations': {
                    'prestation': {
                        'rang': 100,
                        'fiche': {
                            'libelle': 'string', 'code': 'string'
                        },
                        'option': {
                            'libelle': 'string', 'code': 'string'
                        }, 'cas': {
                            'libelle': 'string', 'code': 'string'
                        }
                    }
                }, 'serviceSouscritId': 'string'
            }
        }
        self.assertDictEqual(reponse.json(), resultat_attendu)


"""
Django settings for djangoProject project.

Generated by 'django-admin startproject' using Django 4.2.2.

For more information on this file, see
https://docs.djangoproject.com/en/4.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/4.2/ref/settings/
"""
import os
from pathlib import Path

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/4.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.environ['DJANGO_SECRET']

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = os.getenv('DEBUG', False)

HOMOLOGATION = os.getenv('HOMOLOGATION', False)

ALLOWED_HOSTS = ['*']


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'webservices_enedis',
    'proxy',
    'django_admin_generator',
    'rest_framework',
    'django_extensions',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'proxy.middleware_trace_activite.EnregistreurActivite',
]

ROOT_URLCONF = 'webservices_enedis.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [BASE_DIR / 'templates']
        ,
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'webservices_enedis.wsgi.application'

WS_ENEDIS_LOGIN = os.environ['WS_ENEDIS_LOGIN']
WS_ENEDIS_CONTRAT_ID = os.environ['WS_ENEDIS_CONTRAT_ID']
WS_ENEDIS_URL = os.environ['WS_ENEDIS_URL']

# Database
# https://docs.djangoproject.com/en/4.2/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": os.environ['PROXY_DB_NAME'],
        "USER": os.environ['PROXY_DB_USER'],
        "PASSWORD": os.environ['PROXY_DB_PASSWORD'],
        "HOST": os.environ['PROXY_DB_HOST'],
        "PORT": os.environ['PROXY_DB_PORT'],
    }
}


# Password validation
# https://docs.djangoproject.com/en/4.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/4.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'Europe/Paris'

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/4.2/howto/static-files/

STATIC_URL = 'static/'

# Default primary key field type
# https://docs.djangoproject.com/en/4.2/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

IS_DEV_CONTEXT = (os.environ['ENVIRONEMENT_EXECUTION'] == 'dev')

if not IS_DEV_CONTEXT:
    KEY_PRIVATE_CLIENT_CERTIFICATE = os.environ['KEY_PRIVATE_CLIENT_CERTIFICATE']
    BUCKET_NAME = os.environ['BUCKET_NAME']
    KEY_PUBLIC_CLIENT_CERTIFICATE = os.environ['KEY_PUBLIC_CLIENT_CERTIFICATE']

REST_FRAMEWORK = {
    'DEFAULT_THROTTLE_CLASSES': [
        'rest_framework.throttling.ScopedRateThrottle',
    ],
    'DEFAULT_THROTTLE_RATES': {
        'rechercherpointv2': '1/second',
        'rechercherservicessouscritsmesuresv1': '1/second',
        'consultationpointv4': '1/second',
        'consultationmesuresdetailleesv3': '1/second',
        'consultationmesuresv1': '1/second',
        'consultationdonneestechniquescontractuellesv1': '1/second',
        # 'commandetransmissionhistoriquemesuresv1': '1/second',
        'commandecollectepublicationmesuresv3': '1/second',
        'commandearretservicessouscritsmesuresv1': '1/second',
        'commandeaccesservicessouscritsv1': '1/second',
        'commandem023historiquedonneesmesuresfinesv1': '1/second',
        'commandem023informationstechniquesetcontractuellesv1': '1/second',
        'commandem023mesuresfacturantesv1': '1/second',
    }
}


import logging

from zeep.helpers import serialize_object

from enedis.connection.enedis_connection import EnedisConnection
from enedis.data_models.consultermesuresdetaillees import \
    DemandeConsulterMesuresDetaillees

logger = logging.getLogger(__name__)

class ConsulterMesuresDetaillees:
    name = 'ADAM.ConsulterMesuresServiceReadV3'
    ns = 'http://www.enedis.fr/sge/b2b/services/consultationmesuresdetaillees/common'
    tn_binding = 'http://www.enedis.fr/sge/b2b/services/consultationmesuresdetaillees/v3.0'
    port_binding = 'AdamConsultationMesuresServiceReadHttpBinding'
    service_path = 'ConsultationMesuresDetaillees/v3.0'

    @classmethod
    def consulter_mesures_detaillees(cls, connection: EnedisConnection, donnees: DemandeConsulterMesuresDetaillees, send=True):
        message, response = None, None
        try:
            service, factory, client = connection.soap_service(cls.name, cls.ns, cls.port_binding, cls.service_path,
                                                           cls.tn_binding)

            donnees['initiateurLogin'] = connection.login
            message = factory.ConsulterMesuresDetailleesV3Type(donnees)
            if send:
                response = serialize_object(service.consulterMesuresDetailleesV3(message), dict)

            return serialize_object(message, dict), response
        except Exception as e:
            logger.exception("Erreur appel service")
            return message, {'erreur': str(e.args), 'status': 'failed'}



import logging
from pprint import pprint

from zeep.helpers import serialize_object

from enedis.connection.enedis_connection import EnedisConnection

logger = logging.getLogger(__name__)


class M023Base:
    name = None
    ns = None
    tn_binding = None
    port_binding = None
    service_path = None
    soap_action = None

    @classmethod
    def m023(cls, connection: EnedisConnection, donnees, send=True):
        message, response = None, None
        try:
            service, factory, client = connection.soap_service(cls.name, cls.ns, cls.port_binding, cls.service_path,
                                                           cls.tn_binding)

            donnees['donneesGenerales']['contratId'] = connection.contract_id
            donnees['donneesGenerales']['initiateurLogin'] = connection.login

            if not cls.is_valid_request(donnees):
                raise ValueError('Vous avez fait le mauvais type de demande pour ce service')

            message = cls.create_factory_message(factory, **donnees)

            if send:
                soap_method = getattr(service, cls.soap_action)
                response_obj = soap_method(donneesGenerales=message['donneesGenerales'], demande=message['demande'])
                response = serialize_object(response_obj, dict)
            return serialize_object(message, dict), response
        except Exception as e:
            logger.exception("Erreur appel service")
            return message, {'erreur': str(e.args), 'status': 'failed'}

    @staticmethod
    def is_valid_request(donnees):
        # Logique de validation à implémenter pour chaque type de demande
        raise NotImplementedError

    @staticmethod
    def create_factory_message(_factory, **kwargs):
        # Logique de création du message pour chaque demande
        raise NotImplementedError

class M023HistoriqueMesuresFines(M023Base):
    name = 'B2B_M023MFI'
    ns = 'https://sge-b2b.enedis.fr/services/commandehistoriquedonneesmesuresfines/v1'
    tn_binding = 'https://sge-b2b.enedis.fr/services/commandehistoriquedonneesmesuresfines/v1'
    port_binding = 'AdamCommandeHistoriqueDonneesMesuresFinesBinding'
    service_path = 'CommandeHistoriqueDonneesMesuresFines/v1.0'
    soap_action = 'commandeHistoriqueDonneesMesuresFines'

    @classmethod
    def is_valid_request(cls, donnees):
        return "mesuresTypeCode" in donnees['demande']

    @classmethod
    def create_factory_message(cls, _factory, **kwargs):
        return _factory.demandePublicationMesuresFines(**kwargs)


class M023MesuresFacturantes(M023Base):
    name = 'B2B_M023MFA'
    ns = 'https://sge-b2b.enedis.fr/services/commandehistoriquedonneesmesuresfacturantes/v1'
    tn_binding = 'https://sge-b2b.enedis.fr/services/commandehistoriquedonneesmesuresfacturantes/v1'
    port_binding = 'AdamCommandeHistoriqueDonneesMesuresFacturantesBinding'
    service_path = 'CommandeHistoriqueDonneesMesuresFacturantes/v1.0'
    soap_action = 'commandeHistoriqueDonneesMesuresFacturantes'

    @classmethod
    def is_valid_request(cls, donnees):
        return "mesuresTypeCode" not in donnees['demande'] and 'dateDebut' in donnees['demande']

    @classmethod
    def create_factory_message(cls, _factory, **kwargs):
        return _factory.demandePublicationMesuresFacturantes(**kwargs)


class M023InformationsTechniquesEtContractuelles(M023Base):
    name = 'B2B_M023ITC'
    ns = 'https://sge-b2b.enedis.fr/services/commandeinformationstechniquesetcontractuelles/v1'
    tn_binding = 'https://sge-b2b.enedis.fr/services/commandeinformationstechniquesetcontractuelles/v1'
    port_binding = 'AdamCommandeInformationsTechniquesEtContractuellesBinding'
    service_path = 'CommandeInformationsTechniquesEtContractuelles/v1.0'
    soap_action = 'commandeInformationsTechniquesEtContractuelles'

    @classmethod
    def is_valid_request(cls, donnees):
        return "dateDebut" not in donnees['demande']

    @classmethod
    def create_factory_message(cls, _factory, **kwargs):
        return _factory.demandePublicationITC(**kwargs)




from pathlib import Path


SERVICES = {x.stem: x.resolve().as_posix() for x in Path('./ressources').glob("**/*.wsdl")}

def wsdl(service_name: str) -> str:
    """Return path to WSDL file for `service_name`."""
    try:
        return SERVICES[service_name]
    except KeyError:
        raise KeyError(
            "Unknown service name {!r}, available services are {}".format(
                service_name,
                ", ".join(sorted(SERVICES)),
            ),
        ) from None





import enum

FACTORY_DATA_SIMPLE = 'http://www.enedis.fr/sge/b2b/dictionnaire/v5.0/ds'


class PasCdc(enum.Enum):
    DIX_MINUTES = {'valeur': 10, 'unite': 'min'}
    TRENTE_MINUTES = {'valeur': 30, 'unite': 'min'}


class DomaineTension(enum.Enum):
    BTINF = 'BTINF'
    BTSUP = 'BTSUP'
    HTA = 'HTA'
    HTB = 'HTB'


class ClientFinalCategorieCode(enum.Enum):
    PRO = 'PRO'
    RES = 'RES'


class GrandeurPhysique(enum.Enum):
    PA = 'PA'
    PRI = 'PRI'
    PRC = 'PRC'
    E = 'E'
    TOUT = 'TOUT'
    PMA = 'PMA'
    EA = 'EA'


class MesuresPas(enum.Enum):
    JOUR = 'P1D'
    MOIS = 'P1M'


class GrandeursPhysiquesConsultationv3(enum.Enum):
    PUISSANCE_ACTIVE = 'PA'
    PUISSANCE_REACTIVE_INDUCTIVE = 'PRI'
    PUISSANCE_REACTIVE_CAPACITIVE = 'PRC'
    TENSION = 'E'
    TOUT = 'TOUT'
    PUISSANCES_MAX = 'PMA'
    ENERGIE_QUOTIDIENNE = 'EA'
    ENERGIE_ACTIVE = 'EA'
    ENERGIE_REACTIVE = 'ER'
    ENERGIE_REACTIVE_CAPACITIVE = 'ERC'
    ENERGIE_REACTIVE_INDUCTIVE = 'ERI'
    DUREE_DE_DEPASSEMENT = 'DD'
    DEPASSEMENT_ENERGETIQUE = 'DE'
    DEPASSEMENT_QUADRATIQUE = 'DQ'
    PUISSANCE_MAX_ATTEINTE = 'PMAX'
    TEMPS_DE_FONCTIONNEMENT = 'TF'


class PasMesures(enum.Enum):
    QUOTIDIEN = 'P1D'
    MENSUEL = 'P1M'
    P10MIN = 'PT10M'  # only for C1 C2 C3 C4 and P1 P2 P3
    P30MIN = 'PT30M'  # only for C5 and P4


class PeriodiciteTransmission(enum.Enum):
    QUOTIDIENNE = 'P1D'
    HEBDOMADAIRE = 'P7D'
    MENSUELLE = 'P1M'


class TypeDonneesV1(enum.Enum):
    CDC = 'CDC'
    INDEX = 'IDX'
    PMAX = 'PMAX'
    ENERGIE = 'ENERGIE'


class TypeDonnees(enum.Enum):
    CDC = 'COURBE'
    INDEX = 'INDEX'
    PMAX = 'PMAX'
    ENERGIE = 'ENERGIE'


class TypeDonneesM023(enum.Enum):
    CDC = 'COURBES'
    INDEX = 'INDEX'
    PMAX = 'PMAX'
    ENERGIE = 'ENERGIE'

class SensMesure(enum.Enum):
    INJECTION = 'INJECTION'
    SOUTIRAGE = 'SOUTIRAGE'


class CadreAcces(enum.Enum):
    ACCORD_CLIENT = "ACCORD_CLIENT"
    SERVICE_ACCES = "SERVICE_ACCES"
    EST_TITULAIRE = "EST_TITULAIRE"


class TypeSite(enum.Enum):
    SOUTIRAGE = "soutirage"
    INJECTION = "injection"


class FormatsM023(enum.Enum):
    JSON = "JSON"
    CSV = 'CSV'




from django.test import TestCase
from rest_framework.test import APIClient


class TestViewRechercheServicessouscritsMesures(TestCase):
    maxDiff = None

    def test_view_rechercherservicessouscritsmesures(self):
        client = APIClient()

        entree = {'serveur': 'poste_travail', 'donnees': {'criteres': {'pointId': '12345678911234'}}}
        reponse = client.post('/ws/v1.0/rechercheservicessouscritsmesures/', entree, format='json')
        resultat_attendu = {
            'header': {
                'acquittement': None
            },
            'body': {
                'servicesSouscritsMesures': {
                    'serviceSouscritMesures': [
                        {
                            'serviceSouscritId': 'string', 'pointId': 'string',
                            'serviceSouscritType': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'serviceSouscritLibelle': 'string', 'injection': False, 'soutirage': False,
                            'contratId': 'string',
                            'contratLibelle': 'string', 'etatCode': 'string', 'dateDebut': '2005-09-02',
                            'dateFin': '2019-08-06',
                            'motifFinLibelle': 'string', 'mesuresTypeCode': 'string', 'mesuresPas': 'string',
                            'mesuresCorrigees': False, 'periodiciteTransmission': 'string'
                        }
                    ]
                }
            }
        }
        self.assertDictEqual(reponse.json(), resultat_attendu)



from __future__ import annotations

import datetime as dt
import os

from enedis.utils.regex import verification_prm
from .const import CadreAcces, SensMesure, FormatsM023, TypeDonneesM023

MAX_PRMS = 1500

class DemandeM023(dict):
    def __init__(self, *args, **kwargs):
        _kwargs = {"donneesGenerales": {"initiateurLogin": "", "contratId": ""}, "demande": {}}
        dict.__init__(self, **_kwargs)

    def check_dates(self):
        if not bool(os.getenv('HOMOLOGATION', False)):
            if ((self['demande'] is not None) and ("dateDebut" in self['demande'])
                    and (self['demande']['dateDebut'] > self['demande']['dateFin'])):
                raise ValueError(
                    'incohérence temporelle au niveau des dates. la date de fin est supérieure à la date du jour ou '
                    'antérieure à la date de début')


class DemandeHistoriqueMesuresFines(DemandeM023):
    def __init__(self,
                 date_debut: dt.date,
                 date_fin: dt.date,
                 cadre_acces: CadreAcces,
                 sens: SensMesure,
                 mesure_type_code: TypeDonneesM023,
                 points: [str],
                 format: FormatsM023,
                 mesures_corrigees: bool = None):
        super().__init__()

        if len(points) == 0:
            raise ValueError('il faut renseigner au moins un PRM')
        elif len(points) > MAX_PRMS:
            raise ValueError('Trop de prms demandés')
        else:
            _kwargs = {"pointIds": [], "mesuresTypeCode": mesure_type_code.value, "sens": sens.value,
                       "cadreAcces": cadre_acces.value,
                       "dateDebut": date_debut, "dateFin": date_fin, "format": format.value}
            for point in points:
                verification_prm(point)
                _kwargs['pointIds'].append({'pointId': point})
            if mesures_corrigees is not None:
                _kwargs['mesuresCorrigees'] = mesures_corrigees
            self['demande'] = _kwargs
        self.check_dates()


class DemandeHistoriqueDonneesFacturantes(DemandeM023):
    def __init__(self, date_debut: dt.date, date_fin: dt.date, cadre_acces: CadreAcces, sens: SensMesure,
                 points: [str], format: FormatsM023):
        super().__init__()
        if len(points) == 0:
            raise ValueError('il faut renseigner au moins un PRM')
        elif len(points) > MAX_PRMS:
            raise ValueError('Trop de prms demandés')
        else:
            _kwargs = {"pointIds": [], "sens": sens.value, "cadreAcces": cadre_acces.value, "dateDebut": date_debut,
                       "dateFin": date_fin, "format": format.value}
            for point in points:
                verification_prm(point)
                _kwargs['pointIds'].append({'pointId': point})
            self['demande'] = _kwargs
        self.check_dates()


class DemandeInformationsTechniquesEtContractuelles(DemandeM023):
    def __init__(self, cadre_acces: CadreAcces, sens: SensMesure, points: [str], format: FormatsM023) -> object:
        super().__init__()
        if len(points) == 0:
            raise ValueError('il faut renseigner au moins un PRM')
        elif len(points) > MAX_PRMS:
            raise ValueError('Trop de prms demandés')
        else:
            _kwargs = {"pointIds": [], "sens": sens.value, "cadreAcces": cadre_acces.value, "format": format.value}
            for point in points:
                verification_prm(point)
                _kwargs['pointIds'].append({'pointId': point})
            self['demande'] = _kwargs




from time import sleep

from django.test import TestCase
from rest_framework.test import APIClient

from enedis.data_models.const import DomaineTension, ClientFinalCategorieCode
from enedis.data_models.recherchepoint import AdresseInstallation, Criteres, RechercherPoint


class TestViewRecherchePoint(TestCase):
    def test_view_rechercherpoint(self):
        sleep(1)
        client = APIClient()

        adresse_installation = AdresseInstallation('escalier 8 etage 9 appartement 10', 'batiment E',
                                                   '404 voie introuvable',
                                                   'Faron', '83000', '48414956515')
        criteres = Criteres('12345678901234', '963852', DomaineTension.HTA, 'societe de test',
                            ClientFinalCategorieCode.PRO, True, adresse_installation)
        donnees = RechercherPoint(criteres)
        donnees_entree = {'serveur': 'poste_travail', 'donnees': donnees}
        reponse = client.post('/ws/v2.0/recherchepoint/', donnees_entree, format='json')
        body_attendu = {'points':
                            {
                                'point': [
                                    {
                                        'adresseInstallationNormalisee': {
                                            'ligne1': 'string', 'ligne2': 'string', 'ligne3': 'string', 'ligne4': 'string',
                                            'ligne5': 'string', 'ligne6': 'string', 'ligne7': 'string'
                                        },
                                        'matricule': ['string'], 'numeroSerie': ['string'],
                                        'typeComptage': {
                                            'libelle': 'string', 'code': 'string'
                                        },
                                        'etatContractuel': {
                                            'libelle': 'string', 'code': 'SERVC'
                                        },
                                        'nomClientFinalOuDenominationSociale': 'string', 'id': 'string'
                                    }
                                ]
                            }
        }

        body = reponse.json()['body']
        if 'code' in body['points']['point'][0]['etatContractuel']:
            body['points']['point'][0]['etatContractuel']['code'] = "SERVC"

        self.assertDictEqual(body, body_attendu)




from __future__ import annotations


class PersonnePhysique(dict):
    def __init__(self, nom: str, civilite: str = None, prenom: str = None):
        _kwargs = {'nom': nom}
        if civilite is not None:
            _kwargs['civilite'] = civilite
        if prenom is not None:
            _kwargs['prenom'] = prenom
        dict.__init__(self, **_kwargs)


class PersonneMorale(dict):
    def __init__(self, denomination_sociale: str):
        _kwargs = {'denominationSociale': denomination_sociale}
        dict.__init__(self, **_kwargs)


class DeclarationAccordClient(dict):
    def __init__(self, personne_physique_ou_morale: PersonnePhysique | PersonneMorale, accord=False):
        _kwargs = {'accord': accord}
        if isinstance(personne_physique_ou_morale, PersonnePhysique):
            _kwargs['personnePhysique'] = personne_physique_ou_morale
        elif isinstance(personne_physique_ou_morale, PersonneMorale):
            _kwargs['personneMorale'] = personne_physique_ou_morale
        else:
            raise ValueError('un objet de type PersonnePhysique ou PersonneMorale est attendu en premier argument')
        dict.__init__(self, **_kwargs)


import logging
import os
import re
from threading import Lock
from typing import Tuple, Any

from django.conf import settings
from lxml import etree
from requests import Session
from zeep import Settings, Client, Plugin
from zeep.proxy import OperationProxy
from zeep.transports import Transport

from enedis.utils.wsdl import wsdl

logger = logging.getLogger(__name__)

class SingletonMeta(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            instance = super().__call__(*args, **kwargs)
            cls._instances[cls] = instance
        return cls._instances[cls]

class FixMalformedDatesPlugin(Plugin):
    def ingress(self, envelope, http_headers, operation):
        # Convert the lxml tree to a string for manipulation
        xml_string = etree.tostring(envelope, encoding="unicode")
        # Fix the malformed date
        fixed_xml = re.sub(r'(\d{4}-\d{2}-\d{2})\+00:00', r'\1', xml_string)
        # Return the modified XML as an lxml element
        envelope = etree.fromstring(fixed_xml)
        return envelope, http_headers


class EnedisConnection(metaclass=SingletonMeta):

    def __init__(self, enedis_url: str, login: str, contract_id: str, path_public_cert=None, path_private_cert=None):
        self.__session = None
        self.__transport = None
        self.__is_initialized = False
        self.__lock = Lock()
        self.settings = Settings(strict=True, xml_huge_tree=True)

        # Enedis informations
        self.enedis_url = enedis_url
        self.login = login
        self.contract_id = contract_id
        self.path_public_cert = path_public_cert
        self.path_private_cert = path_private_cert

        # Initialize the connection
        self._initialize_connection()

    def _initialize_connection(self):
        with self.__lock:
            if self.__is_initialized:
                return

            logger.info("Initializing connection...")
            self.__session = Session()

            self.__session.cert = (self.path_public_cert, self.path_private_cert)
            try:
                self.__transport = Transport(session=self.__session)
                self.__is_initialized = True
                logger.info("Connection initialized successfully.")
            except Exception as e:
                logger.error(f"Failed to initialize connection: {e}")
                self._cleanup_resources()
                raise

    def _cleanup_resources(self):
        """
        Cleans up temporary files and session resources.
        """
        logger.info("Cleaning up resources...")
        if self.__session:
            self.__session.close()

        self.__is_initialized = False

    def get_transport(self):
        """
        Returns the transport object. Reinitializes the connection if necessary.
        """
        if not self.__is_initialized:
            logger.warning("Connection was closed or uninitialized. Reinitializing...")
            self._initialize_connection()
        return self.__transport

    def close_connection(self):
        """
        Closes the connection and cleans up resources.
        """
        with self.__lock:
            if self.__is_initialized:
                logger.info("Closing connection...")
                self._cleanup_resources()
                self.__transport = None
                self.__session = None
                self.enedis_url = None
                self.login = None
                self.contract_id = None
                self.path_public_cert = None
                self.path_private_cert = None
                logger.info("Connection closed successfully.")

    def soap_service(self, name: str, ns: str, port_binding: str, service_path: str, tn_binding: str = None) -> Tuple[OperationProxy, Any, Client]:
        """
        Creates a SOAP service proxy for interacting with Enedis API.

        Args:
            name (str): The WSDL name.
            ns (str): Namespace used in the type factory.
            port_binding (str): Port binding name.
            service_path (str): Path to the SOAP service endpoint.
            tn_binding (str, optional): Target namespace binding. Defaults to None.

        Returns:
            Tuple[OperationProxy, DynamicTypeFactory, Client]: A tuple containing:
                - The service proxy to perform operations.
                - The factory to create new data types.
                - The Client object for further use.
        """

        client = Client(wsdl=wsdl(name), transport=self.get_transport(), settings=self.settings,
                        plugins=[FixMalformedDatesPlugin()])

        service_proxy = client.create_service(f'{{{tn_binding}}}{port_binding}', os.path.join(self.enedis_url, service_path))
        print(ns)
        factory = client.type_factory(ns)

        return service_proxy, factory, client

    def __del__(self):
        self.close_connection()



import datetime as dt

from django.test import TestCase
from rest_framework.test import APIClient

from enedis.data_models.const import TypeDonnees, GrandeurPhysique, SensMesure, CadreAcces
from enedis.data_models.consultermesuresdetaillees import \
    DemandeConsulterMesuresDetaillees


class TestViewConsulterMesuresDetaillees(TestCase):
    maxDiff = None

    def test_view_consultermesuresdetaillees(self):
        client = APIClient()

        test_data = {'serveur': 'poste_travail',
                     'donnees': DemandeConsulterMesuresDetaillees('12345678901234', TypeDonnees.CDC,
                                                                  GrandeurPhysique.TOUT,
                                                                  SensMesure.SOUTIRAGE,
                                                                  dt.date(2023, 1, 1), dt.date(2023, 12, 1), False,
                                                                  CadreAcces.ACCORD_CLIENT)
                     }

        reponse = client.post('/ws/v3.0/consultermesuresdetaillees/', test_data, format='json')

        resultat_attendu = {'pointId': 'string', 'mesuresCorrigees': 'BEST',
                            'periode': {
                                'dateDebut': '2013-10-10', 'dateFin': '2019-08-15'},
                            'grandeur': [
                                {
                                    'grandeurMetier': 'PROD', 'grandeurPhysique': 'PA', 'unite': 'string',
                                    'points': [
                                        {
                                            'v': 'string', 'd': 'string', 'p': 'string', 'n': 'string', 'tc': 'string',
                                            'iv': 'string', 'ec': 'string'
                                        }
                                    ]
                                }
                            ],
                            'contexte': [
                                {
                                    'etapeMetier': 'string', 'contexteReleve': 'FMR', 'typeReleve': 'AP',
                                    'motifReleve': 'string',
                                    'grandeur': [
                                        {
                                            'grandeurMetier': 'PROD', 'grandeurPhysique': 'E', 'unite': 'string',
                                            'calendrier': [
                                                {
                                                    'idCalendrier': 'string', 'libelleCalendrier': 'string',
                                                    'classeTemporelle': [
                                                        {
                                                            'idClasseTemporelle': 'string',
                                                            'libelleClasseTemporelle': 'string',
                                                            'codeCadran': 'string',
                                                            'valeur': [
                                                                {
                                                                    'd': 'string', 'v': 100, 'iv': 2
                                                                }
                                                            ]
                                                        }
                                                    ]
                                                }
                                            ],
                                            'cadranTotalisateur': {
                                                'codeCadran': 'string',
                                                'valeur': [
                                                    {
                                                        'd': 'string', 'v': 100, 'iv': 2
                                                    }
                                                ]
                                            }
                                        }
                                    ]
                                }
                            ], 'typeValeur': 'string', 'modeCalcul': 'string', 'pas': 'string'
                            }
        self.assertDictEqual(resultat_attendu, reponse.json())



import datetime as dt
from unittest.mock import patch

import zeep
from zeep.helpers import serialize_object

from enedis.data_models.rechercheservicesouscritmesures import RechercherServicesSouscritsMesures, Criteres
from enedis.services.recherche_services_souscrits_mesures import RechercheServicesSouscritsMesures
from proxy.tests.services.ServicesTestCase import raise_transport_errror, raise_creation_error, ServicesTestCase


class TestRecherche(ServicesTestCase):
    maxDiff = None
    def test_creation_message(self):
        donnees = RechercherServicesSouscritsMesures(Criteres('12345678911234'))
        msg, _ = RechercheServicesSouscritsMesures.rechercher_services_souscrit_mesures(self.connection, donnees, send=False)
        self.assertDictEqual(msg, {'criteres': {'contratId': '1111111', 'pointId': '12345678911234'},
                                   'loginUtilisateur': 'test@test.fr'})

    def test_connexion(self):
        donnees = RechercherServicesSouscritsMesures(Criteres('12345678911234'))

        _, response = RechercheServicesSouscritsMesures.rechercher_services_souscrit_mesures(self.connection, donnees)

        resultat_attendu = {
            'header': {
                'acquittement': None
            },
            'body': {
                'servicesSouscritsMesures': {
                    'serviceSouscritMesures': [
                        {
                            'serviceSouscritId': 'string', 'pointId': 'string',
                            'serviceSouscritType': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'serviceSouscritLibelle': 'string', 'injection': False, 'soutirage': False, 'contratId': 'string',
                            'contratLibelle': 'string', 'etatCode': 'string', 'dateDebut': dt.date(2005, 9, 2), 'dateFin': dt.date(2019, 8, 6),
                            'motifFinLibelle': 'string', 'mesuresTypeCode': 'string', 'mesuresPas': 'string',
                            'mesuresCorrigees': False, 'periodiciteTransmission': 'string'
                        }
                    ]
                }
            }
        }

        self.assertDictEqual(response, resultat_attendu)

    @patch.object(zeep.helpers, 'serialize_object', raise_transport_errror)
    def test_connexion_echoue(self):
        donnees = RechercherServicesSouscritsMesures(Criteres('12345678911234'))
        _, response = RechercheServicesSouscritsMesures.rechercher_services_souscrit_mesures(self.connection, donnees)
        self.assertEqual(response, {'erreur': "('une erreur inconnue est survenue',)", 'status': 'failed'})









from pprint import pprint

from django.test import TestCase
from rest_framework.test import APIClient

from enedis.data_models.M023 import DemandeInformationsTechniquesEtContractuelles, DemandeM023
from enedis.data_models.const import FormatsM023, SensMesure, CadreAcces
from enedis.data_models.consulterdonneestechniquescontractuelles import ConsulterDonneesTechniquesContractuelles


class TestViewConsulterDonneesTechniquesContractuelles(TestCase):
    maxDiff = None

    def test_view_consulterdonneestechniquescontractuelles(self):
        client = APIClient()
        datas = {'serveur': 'poste_travail', 'donnees': ConsulterDonneesTechniquesContractuelles('12345678901234', autorisation_client=True)}
        reponse = client.post('/ws/v1.0/consulterdonneestechniquescontractuelles/', datas, format='json')
        expected_result = dict(header={'acquittement': None},
                               body={
                                   'point': {
                                       'donneesGenerales': {'etatContractuel': {'libelle': 'string', 'code': 'RESIL'},
                                                            'adresseInstallation': {
                                                                'escalierEtEtageEtAppartement': 'string',
                                                                'batiment': 'string',
                                                                'numeroEtNomVoie': 'string',
                                                                'lieuDit': 'string',
                                                                'codePostal': 'string',
                                                                'commune': {'libelle': 'string', 'code': 'string'}},
                                                            'dateDerniereModificationFormuleTarifaireAcheminement': '2011-09-12',
                                                            'dateDerniereAugmentationPuissanceSouscrite': '2001-09-05',
                                                            'dateDerniereDiminutionPuissanceSouscrite': '2018-10-15',
                                                            'segment': {'libelle': 'string', 'code': 'C3'},
                                                            'niveauOuvertureServices': 'string'},
                                       'situationAlimentation': {
                                           'alimentationPrincipale': {
                                               'domaineTension': {'libelle': 'string', 'code': 'BTSUP'},
                                               'tensionLivraison': {'libelle': 'string', 'code': 'string'},
                                               'modeApresCompteur': {'libelle': 'string', 'code': 'string'},
                                               'puissanceRaccordementSoutirage': {'valeur': 1000.0,
                                                                                  'unite': 'kVAR'}}},
                                       'situationComptage': {
                                           'dispositifComptage': {
                                               'typeComptage': {'libelle': 'string', 'code': 'string'}, 'compteurs': {
                                                   'compteur': [
                                                       {'localisation': {'libelle': 'string', 'code': 'string'},
                                                        'matricule': 'string',
                                                        'ticActivee': True, 'ticStandard': True, 'ticActivable': False,
                                                        'plagesHeuresCreuses': 'string',
                                                        'parametresTeleAcces': {'numeroTelephone': 'string',
                                                                                'numeroVoieAiguillage': 100,
                                                                                'etatLigneTelephonique': 'string',
                                                                                'fenetreEcouteClient': {
                                                                                    'heureDebut': "12:41:25+00:00",
                                                                                    'duree': {'valeur': 1000.0,
                                                                                              'unite': 'annee'}
                                                                                },
                                                                                'cle': 'string'},
                                                        'programmationHoraire': {
                                                            'programmationPosteHoraire': [
                                                                {'libelle': 'string', 'periodesHoraires': 'string',
                                                                 'code': 'string'}]}}]},
                                               'disjoncteur': {'calibre': {'libelle': 'string', 'code': 'string'}},
                                               'relais': {'plageHeuresCreuses': 'string'},
                                               'transformateurCourant': {
                                                   'calibre': {'libelle': 'string', 'code': 'string'},
                                                   'couplage': {'libelle': 'string', 'code': 'string'},
                                                   'classePrecision': {'libelle': 'string',
                                                                       'code': 'string'},
                                                   'position': {'libelle': 'string', 'code': 'string'}},
                                               'transformateurTension': {
                                                   'calibre': {'libelle': 'string', 'code': 'string'},
                                                   'couplage': {'libelle': 'string', 'code': 'string'},
                                                   'classePrecision': {'libelle': 'string',
                                                                       'code': 'string'}}},
                                           'caracteristiquesReleve': {
                                               'modeTraitement': {'libelle': 'string', 'code': 'string'},
                                               'periodicite': {'libelle': 'string', 'code': 'string'},
                                               'plageReleve': {'libelle': 'string', 'code': 'string'}},
                                           'modeReleve': {'libelle': 'string', 'code': 'string'},
                                           'mediaReleve': {'libelle': 'string', 'code': 'string'},
                                           'futuresPlagesHeuresCreuses': {'libelle': 'string', 'code': 'string'},
                                           'futuresProgrammationsHoraires': {
                                               'formuleTarifaireAcheminement': [{'libelle': 'string',
                                                                                 'programmationHoraire': {
                                                                                     'programmationPosteHoraire': [
                                                                                         {'libelle': 'string',
                                                                                          'periodesHoraires': 'string',
                                                                                          'code': 'string'}]},
                                                                                 'code': 'string'}]}},
                                       'situationContractuelle': {
                                           'structureTarifaire': {
                                               'formuleTarifaireAcheminement': {'libelle': 'string', 'code': 'string'},
                                               'longueUtilisation': {
                                                   'contexte': {'libelle': 'string', 'code': 'string'},
                                                   'forfait': {'valeur': 1000.0,
                                                               'unite': 's'}},
                                               'puissanceSouscriteMax': {'valeur': 1000.0,
                                                                         'unite': 'kW'}, 'denivelePuissances': {
                                                   'classesTemporelles': {
                                                       'classeTemporelle': [{'libelle': 'string', 'puissance': {
                                                           'valeur': 1000.0, 'unite': 'kVAR'}, 'code': 'string'}]}},
                                               'calendrierFrn': {'libelle': 'string', 'code': 'string'}}},
                                       'id': 'string'}})

        self.assertEqual(expected_result, reponse.json())


import logging

from zeep.helpers import serialize_object

from enedis.connection.enedis_connection import EnedisConnection
from enedis.data_models.commandercollectepublicationmesures import DemandeCollectePublicationMesure

logger = logging.getLogger(__name__)

class CommanderCollectePublicationMesures:
    name = 'CommandeCollectePublicationMesures-v3.0'
    ns = 'http://www.enedis.fr/sge/b2b/commandercollectepublicationmesures/v3.0'
    tn_binding = 'http://www.enedis.fr/sge/b2b/commandecollectepublicationmesures/v3.0'
    port_binding = 'CommandeCollectePublicationMesuresBinding'
    service_path = 'CommandeCollectePublicationMesures/v3.0'

    @classmethod
    def commander_collecte_publication(cls, connection:EnedisConnection, donnees: DemandeCollectePublicationMesure, send=True):
        message, response = None, None
        try:
            service, factory, client = connection.soap_service(cls.name, cls.ns, cls.port_binding, cls.service_path,
                                                           cls.tn_binding)

            donnees['donneesGenerales']['initiateurLogin'] = connection.login
            donnees['donneesGenerales']['contratId'] = connection.contract_id

            message = factory.CommanderCollectePublicationMesuresType(donnees)
            if send:
                response = serialize_object(service.commanderCollectePublicationMesures(message), dict)

            return serialize_object(message, dict), response

        except Exception as e:
            logger.exception("Erreur appel service")
            return message, {'erreur': str(e.args), 'status': 'failed'}



from .const import DomaineTension, ClientFinalCategorieCode


class AdresseInstallation(dict):
    def __init__(self, escalier_etage_et_appartement: str = None, batiment: str = None, numero_et_nom_voie: str = None,
                 lieu_dit: str = None, code_postal: str = None, code_insee_commune: str = None):
        _kwargs = {}
        if escalier_etage_et_appartement is not None:
            _kwargs['escalierEtEtageEtAppartement'] = escalier_etage_et_appartement
        if batiment is not None:
            _kwargs['batiment'] = batiment
        if numero_et_nom_voie is not None:
            _kwargs['numeroEtNomVoie'] = numero_et_nom_voie
        if lieu_dit is not None:
            _kwargs['lieuDit'] = lieu_dit
        if code_postal is not None:
            _kwargs['codePostal'] = code_postal
        if code_insee_commune is not None:
            _kwargs['codeInseeCommune'] = code_insee_commune
        dict.__init__(self, **_kwargs)


class Criteres(dict):
    def __init__(self, num_siret: str = None, matricule_ou_numero_serie: str = None,
                 domaine_tension_alimentation_code: DomaineTension = None,
                 nom_client_final_ou_denomination_sociale: str = None,
                 categorie_client_final_code: ClientFinalCategorieCode = None, recherche_hors_perimetre: bool = None,
                 adresse_installation: AdresseInstallation = None):
        _kwargs = {}
        if num_siret is not None:
            _kwargs['numSiret'] = num_siret
        if matricule_ou_numero_serie is not None:
            _kwargs['matriculeOuNumeroSerie'] = matricule_ou_numero_serie
        if domaine_tension_alimentation_code is not None:
            _kwargs['domaineTensionAlimentationCode'] = domaine_tension_alimentation_code.value
        if nom_client_final_ou_denomination_sociale is not None:
            _kwargs['nomClientFinalOuDenominationSociale'] = nom_client_final_ou_denomination_sociale
        if categorie_client_final_code is not None:
            _kwargs['categorieClientFinalCode'] = categorie_client_final_code.value
        if recherche_hors_perimetre is not None:
            _kwargs['rechercheHorsPerimetre'] = recherche_hors_perimetre
        if adresse_installation is not None:
            _kwargs['adresseInstallation'] = adresse_installation
        dict.__init__(self, **_kwargs)

class RechercherPoint(dict):
    def __init__(self, criteres: Criteres):
        dict.__init__(self, criteres=criteres, loginUtilisateur='')


from pprint import pprint

from enedis.data_models.M023 import *
from enedis.data_models.const import *
from enedis.services.commande_M023 import M023InformationsTechniquesEtContractuelles, M023MesuresFacturantes, M023HistoriqueMesuresFines
from proxy.tests.services.ServicesTestCase import ServicesTestCase


class TestM023(ServicesTestCase):
    maxDiff = None

    def test_connexion(self):
        donnees = DemandeInformationsTechniquesEtContractuelles(CadreAcces.ACCORD_CLIENT,
                                                                SensMesure.SOUTIRAGE, ['12345678911234'],
                                                                FormatsM023.JSON)
        msg, response = M023InformationsTechniquesEtContractuelles.m023(self.connection, donnees)
        pprint(msg)
        self.assertDictEqual(msg, {'demande': {'cadreAcces': 'ACCORD_CLIENT',
                                               'format': 'JSON',
                                               'pointIds': [{'pointId': '12345678911234'}],
                                               'sens': 'SOUTIRAGE'},
                                   'donneesGenerales': {'affaireReference': None,
                                                        'contratId': '1111111',
                                                        'initiateurLogin': 'test@test.fr',
                                                        'referenceDemandeur': None,
                                                        'referenceRegroupement': None}}
                             )
        self.assertEqual(response, 'string')

        donnees = DemandeHistoriqueDonneesFacturantes(dt.date(2023, 1, 1),
                                                      dt.date(2023, 6, 1),
                                                      CadreAcces.ACCORD_CLIENT,
                                                      SensMesure.SOUTIRAGE,
                                                      ['12345678911234'],
                                                      FormatsM023.CSV)
        msg, response = M023MesuresFacturantes.m023(self.connection, donnees)
        self.assertDictEqual(msg, {'demande': {'cadreAcces': 'ACCORD_CLIENT',
                                               'dateDebut': dt.date(2023, 1, 1),
                                               'dateFin': dt.date(2023, 6, 1),
                                               'format': 'CSV',
                                               'pointIds': [{'pointId': '12345678911234'}],
                                               'sens': 'SOUTIRAGE'},
                                   'donneesGenerales': {'affaireReference': None,
                                                        'contratId': '1111111',
                                                        'initiateurLogin': 'test@test.fr',
                                                        'referenceDemandeur': None,
                                                        'referenceRegroupement': None}})
        self.assertEqual(response, 'string')

        donnees = DemandeHistoriqueMesuresFines(dt.date(2023, 1, 1),
                                                dt.date(2023, 6, 1),
                                                CadreAcces.ACCORD_CLIENT,
                                                SensMesure.SOUTIRAGE,
                                                TypeDonneesM023.CDC,
                                                ['12345678911234'],
                                                FormatsM023.JSON,
                                                True)

        msg, response = M023HistoriqueMesuresFines.m023(self.connection, donnees)
        self.assertDictEqual(msg, {'demande': {'cadreAcces': 'ACCORD_CLIENT',
                                               'dateDebut': dt.date(2023, 1, 1),
                                               'dateFin': dt.date(2023, 6, 1),
                                               'format': 'JSON',
                                               'mesuresCorrigees': True,
                                               'mesuresTypeCode': 'COURBES',
                                               'pointIds': [{'pointId': '12345678911234'}],
                                               'sens': 'SOUTIRAGE'},
                                   'donneesGenerales': {'affaireReference': None,
                                                        'contratId': '1111111',
                                                        'initiateurLogin': 'test@test.fr',
                                                        'referenceDemandeur': None,
                                                        'referenceRegroupement': None}})
        self.assertEqual(response, 'string')

    def test_mauvaises_dates(self):

        self.assertRaises(ValueError, DemandeHistoriqueDonneesFacturantes, dt.date(2023, 2, 1),
                                                      dt.date(2023, 1, 1),
                                                      CadreAcces.ACCORD_CLIENT,
                                                      SensMesure.SOUTIRAGE,
                                                      ['12345678911234'],
                                                      FormatsM023.CSV)


import datetime as dt
import decimal
from pprint import pprint
from unittest.mock import patch

import pytz
import zeep
from dateutil import tz
from zeep.helpers import serialize_object

from enedis.data_models.consulterdonneestechniquescontractuelles import ConsulterDonneesTechniquesContractuelles
from enedis.services.consulter_donnees_techniques_contractuelles import ConsulterDoneesTechniquesContractuelles
from proxy.tests.services.ServicesTestCase import ServicesTestCase, raise_transport_errror, raise_creation_error


def normalize_to_utc(dt):
    if dt.tzinfo is None:
        return dt
    return dt.astimezone(pytz.UTC)

def normalize_dict_timezones(d):
    if isinstance(d, dict):
        return {k: normalize_dict_timezones(v) for k, v in d.items()}
    elif isinstance(d, list):
        return [normalize_dict_timezones(x) for x in d]
    elif isinstance(d, dt.datetime):
        return normalize_to_utc(d)
    elif isinstance(d, dt.time):
        _dt = dt.datetime.combine(dt.date.today(), d)
        utc_dt = _dt.astimezone(tz.gettz('UTC')) if _dt.tzinfo else _dt
        return utc_dt.time()
    else:
        return d

def convert_decimal_to_float(d):
    if isinstance(d, dict):
        return {k: convert_decimal_to_float(v) for k, v in d.items()}
    elif isinstance(d, list):
        return [convert_decimal_to_float(x) for x in d]
    elif isinstance(d, decimal.Decimal):
        return float(d)
    else:
        return d

class TestConsulterDonneesTechniquesContractuelles(ServicesTestCase):
    maxDiff = None

    def test_creation_message_correct(self):
        donnees = ConsulterDonneesTechniquesContractuelles('12345678901234', autorisation_client=True)
        msg, _ = ConsulterDoneesTechniquesContractuelles.consultation_donnees_techniques_contractuelles(self.connection, donnees, send=False)
        expected_message = {'pointId': '12345678901234', 'loginUtilisateur': 'test@test.fr', 'autorisationClient': True}
        self.assertDictEqual(msg, expected_message)

    def test_connexion(self):
        donnees = ConsulterDonneesTechniquesContractuelles('12345678901234', autorisation_client=True)

        msg, response = ConsulterDoneesTechniquesContractuelles.consultation_donnees_techniques_contractuelles(self.connection, donnees)

        expected_response = dict(header={'acquittement': None},
                               body={
                                   'point': {
                                       'donneesGenerales': {'etatContractuel': {'libelle': 'string', 'code': 'RESIL'},
                                                            'adresseInstallation': {
                                                                'escalierEtEtageEtAppartement': 'string',
                                                                'batiment': 'string',
                                                                'numeroEtNomVoie': 'string',
                                                                'lieuDit': 'string',
                                                                'codePostal': 'string',
                                                                'commune': {'libelle': 'string', 'code': 'string'}},
                                                            'dateDerniereModificationFormuleTarifaireAcheminement': dt.date(
                                                                2011, 9, 12),
                                                            'dateDerniereAugmentationPuissanceSouscrite': dt.date(2001,
                                                                                                                  9, 5),
                                                            'dateDerniereDiminutionPuissanceSouscrite': dt.date(2018,
                                                                                                                10, 15),
                                                            'segment': {'libelle': 'string', 'code': 'C3'},
                                                            'niveauOuvertureServices': 'string'},
                                       'situationAlimentation': {
                                           'alimentationPrincipale': {
                                               'domaineTension': {'libelle': 'string', 'code': 'BTSUP'},
                                               'tensionLivraison': {'libelle': 'string', 'code': 'string'},
                                               'modeApresCompteur': {'libelle': 'string', 'code': 'string'},
                                               'puissanceRaccordementSoutirage': {'valeur': 1000.,
                                                                                  'unite': 'kVAR'}}},
                                       'situationComptage': {
                                           'dispositifComptage': {
                                               'typeComptage': {'libelle': 'string', 'code': 'string'}, 'compteurs': {
                                                   'compteur': [
                                                       {'localisation': {'libelle': 'string', 'code': 'string'},
                                                        'matricule': 'string',
                                                        'ticActivee': True, 'ticStandard': True, 'ticActivable': False,
                                                        'plagesHeuresCreuses': 'string',
                                                        'parametresTeleAcces': {'numeroTelephone': 'string',
                                                                                'numeroVoieAiguillage': 100,
                                                                                'etatLigneTelephonique': 'string',
                                                                                'fenetreEcouteClient': {
                                                                                    'heureDebut': dt.time(12, 41, 25),
                                                                                    'duree': {'valeur': 1000.,
                                                                                              'unite': 'annee'}
                                                                                },
                                                                                'cle': 'string'},
                                                        'programmationHoraire': {
                                                            'programmationPosteHoraire': [
                                                                {'libelle': 'string', 'periodesHoraires': 'string',
                                                                 'code': 'string'}]}}]},
                                               'disjoncteur': {'calibre': {'libelle': 'string', 'code': 'string'}},
                                               'relais': {'plageHeuresCreuses': 'string'},
                                               'transformateurCourant': {
                                                   'calibre': {'libelle': 'string', 'code': 'string'},
                                                   'couplage': {'libelle': 'string', 'code': 'string'},
                                                   'classePrecision': {'libelle': 'string',
                                                                       'code': 'string'},
                                                   'position': {'libelle': 'string', 'code': 'string'}},
                                               'transformateurTension': {
                                                   'calibre': {'libelle': 'string', 'code': 'string'},
                                                   'couplage': {'libelle': 'string', 'code': 'string'},
                                                   'classePrecision': {'libelle': 'string',
                                                                       'code': 'string'}}},
                                           'caracteristiquesReleve': {
                                               'modeTraitement': {'libelle': 'string', 'code': 'string'},
                                               'periodicite': {'libelle': 'string', 'code': 'string'},
                                               'plageReleve': {'libelle': 'string', 'code': 'string'}},
                                           'modeReleve': {'libelle': 'string', 'code': 'string'},
                                           'mediaReleve': {'libelle': 'string', 'code': 'string'},
                                           'futuresPlagesHeuresCreuses': {'libelle': 'string', 'code': 'string'},
                                           'futuresProgrammationsHoraires': {
                                               'formuleTarifaireAcheminement': [{'libelle': 'string',
                                                                                 'programmationHoraire': {
                                                                                     'programmationPosteHoraire': [
                                                                                         {'libelle': 'string',
                                                                                          'periodesHoraires': 'string',
                                                                                          'code': 'string'}]},
                                                                                 'code': 'string'}]}},
                                       'situationContractuelle': {
                                           'structureTarifaire': {
                                               'formuleTarifaireAcheminement': {'libelle': 'string', 'code': 'string'},
                                               'longueUtilisation': {
                                                   'contexte': {'libelle': 'string', 'code': 'string'},
                                                   'forfait': {'valeur': 1000.,
                                                               'unite': 's'}},
                                               'puissanceSouscriteMax': {'valeur': 1000.,
                                                                         'unite': 'kW'}, 'denivelePuissances': {
                                                   'classesTemporelles': {
                                                       'classeTemporelle': [{'libelle': 'string', 'puissance': {
                                                           'valeur': 1000., 'unite': 'kVAR'}, 'code': 'string'}]}},
                                               'calendrierFrn': {'libelle': 'string', 'code': 'string'}}},
                                       'id': 'string'}})

        response = convert_decimal_to_float(response)
        response = normalize_dict_timezones(response)
        self.assertDictEqual(response, expected_response)

    @patch.object(zeep.helpers, 'serialize_object', raise_transport_errror)
    def test_connexion_echoue(self):
        donnees = ConsulterDonneesTechniquesContractuelles('12345678901234', autorisation_client=True)

        msg, response = ConsulterDoneesTechniquesContractuelles.consultation_donnees_techniques_contractuelles(self.connection, donnees)
        self.assertEqual(response, {'erreur': "('une erreur inconnue est survenue',)", 'status': 'failed'})






import os
import unittest

import zeep
from django.test import TestCase

from enedis.connection.enedis_connection import EnedisConnection


class ServicesTestCase(TestCase):

    def setUp(self):
        enedis_url = os.getenv('WS_ENEDIS_URL', "http://imposter:8080/")
        self.connection = EnedisConnection(enedis_url, 'test@test.fr', '1111111')


def raise_transport_errror(*args, **kwargs):
    raise zeep.exceptions.TransportError("une erreur inconnue est survenue")


def raise_creation_error(*args, **kwargs):
    raise ValueError('test pour voir si cela fonctionne')



import datetime as dt
from unittest.mock import patch

import zeep
from zeep.helpers import serialize_object

from enedis.data_models.const import *
from enedis.data_models.consultermesuresdetaillees import \
    DemandeConsulterMesuresDetaillees
from enedis.services.consulter_mesures_detaillees import ConsulterMesuresDetaillees
from proxy.tests.services.ServicesTestCase import raise_transport_errror, ServicesTestCase


class TestConsulterMesuresDetaillees(ServicesTestCase):
    maxDiff = None

    def test_creation_message(self):
        donnees = DemandeConsulterMesuresDetaillees('12345678911234', TypeDonnees.CDC, GrandeurPhysique.TOUT, SensMesure.SOUTIRAGE,
                                                    dt.date(2023, 1, 1), dt.date(2023, 12, 1), False, CadreAcces.ACCORD_CLIENT, MesuresPas.JOUR)

        msg, _ = ConsulterMesuresDetaillees.consulter_mesures_detaillees(self.connection, donnees, send=False)
        expected_message = {'demande':
            {'initiateurLogin': 'test@test.fr', 'pointId': '12345678911234', 'mesuresTypeCode': 'COURBE',
             'grandeurPhysique': 'TOUT', 'dateDebut': dt.date(2023, 1, 1), 'dateFin': dt.date(2023, 12, 1),
             'mesuresPas': 'P1D', 'mesuresCorrigees': False, 'sens': 'SOUTIRAGE', 'cadreAcces': 'ACCORD_CLIENT'}}

        self.assertDictEqual(msg, expected_message)

    def test_creation_message_errone(self):
        donnees = DemandeConsulterMesuresDetaillees('12345678911234', TypeDonnees.CDC, GrandeurPhysique.TOUT,
                                                    SensMesure.SOUTIRAGE,
                                                    dt.date.today(), dt.date.today(), None,
                                                    CadreAcces.ACCORD_CLIENT)


        _, response = ConsulterMesuresDetaillees.consulter_mesures_detaillees(self.connection, donnees)
        self.assertEqual(response['erreur'], "('Missing element mesuresCorrigees',)",
                         'attention le programme continue normalement alors qu\'il manque un élément dans les entrées')

    def test_connexion(self):
        donnees = DemandeConsulterMesuresDetaillees('12345678911234', TypeDonnees.CDC, GrandeurPhysique.TOUT,
                                                    SensMesure.SOUTIRAGE,
                                                    dt.date(2023, 1, 1), dt.date(2023, 12, 1), False,
                                                    CadreAcces.ACCORD_CLIENT)

        _, response = ConsulterMesuresDetaillees.consulter_mesures_detaillees(self.connection, donnees)

        expected_response = {'pointId': 'string', 'mesuresCorrigees': 'BEST',
                            'periode': {
                                'dateDebut': dt.date(2013, 10, 10), 'dateFin': dt.date(2019, 8, 15)},
                            'grandeur': [
                                {
                                    'grandeurMetier': 'PROD', 'grandeurPhysique': 'PA', 'unite': 'string',
                                    'points': [
                                        {
                                            'v': 'string', 'd': 'string', 'p': 'string', 'n': 'string', 'tc': 'string',
                                            'iv': 'string', 'ec': 'string'
                                        }
                                    ]
                                }
                            ],
                            'contexte': [
                                {
                                    'etapeMetier': 'string', 'contexteReleve': 'FMR', 'typeReleve': 'AP', 'motifReleve': 'string',
                                    'grandeur': [
                                        {
                                            'grandeurMetier': 'PROD', 'grandeurPhysique': 'E', 'unite': 'string',
                                            'calendrier': [
                                                {
                                                    'idCalendrier': 'string', 'libelleCalendrier': 'string',
                                                    'classeTemporelle': [
                                                        {
                                                            'idClasseTemporelle': 'string', 'libelleClasseTemporelle': 'string',
                                                            'codeCadran': 'string',
                                                            'valeur': [
                                                                {
                                                                    'd': 'string', 'v': 100, 'iv': 2
                                                                }
                                                            ]
                                                        }
                                                    ]
                                                }
                                            ],
                                            'cadranTotalisateur': {
                                                'codeCadran': 'string',
                                                'valeur': [
                                                    {
                                                        'd': 'string', 'v': 100, 'iv': 2
                                                    }
                                                ]
                                            }
                                        }
                                    ]
                                }
                            ], 'typeValeur': 'string', 'modeCalcul': 'string', 'pas': 'string'
                            }

        self.assertDictEqual(response, expected_response)

    @patch.object(zeep.helpers, 'serialize_object', raise_transport_errror)
    def test_connexion_echoue(self):
        donnees = DemandeConsulterMesuresDetaillees('12345678911234', TypeDonnees.CDC, GrandeurPhysique.TOUT,
                                                    SensMesure.SOUTIRAGE,
                                                    dt.date(2023, 1, 1), dt.date(2023, 12, 1), False,
                                                    CadreAcces.ACCORD_CLIENT)

        _, response = ConsulterMesuresDetaillees.consulter_mesures_detaillees(self.connection, donnees)
        self.assertEqual(response, {'erreur': "('une erreur inconnue est survenue',)", 'status': 'failed'})







# vim: set fileencoding=utf-8 :
from django.contrib import admin

import proxy.models as models


class TraceActiviteAdmin(admin.ModelAdmin):

    list_display = (
        'id',
        'demandeur',
        'RAE_concerne',
        'service_concerne',
        'horodate',
    )
    list_filter = (
        'demandeur',
        'service_concerne',
    )


def _register(model, admin_class):
    admin.site.register(model, admin_class)


_register(models.TraceActivite, TraceActiviteAdmin)



import logging

from zeep.helpers import serialize_object

from enedis.connection.enedis_connection import EnedisConnection
from enedis.data_models.consultermesures import DonneesConsulterMesures
from enedis.utils.regex import verification_prm

logger = logging.getLogger(__name__)

class ConsulterMesures:
    name = 'ConsultationMesures-v1.1'
    ns = 'http://www.enedis.fr/sge/b2b/services/consultermesures/v1.1'
    tn_binding = 'http://www.enedis.fr/sge/b2b/services/consultationmesures/v1.1'
    port_binding = 'ConsultationMesuresBinding'
    service_path = 'ConsultationMesures/v1.1'

    @classmethod
    def consulter_mesures(cls, connection:EnedisConnection, donnees: DonneesConsulterMesures, send=True):
        message, response = None, None
        try:
            service, factory, client = connection.soap_service(cls.name, cls.ns, cls.port_binding, cls.service_path,
                                                           cls.tn_binding)

            verification_prm(donnees['pointId'])

            donnees['loginDemandeur'] = connection.login
            donnees['contratId'] = connection.contract_id

            message = factory.ConsulterMesuresType(**donnees)

            if send:
                response = serialize_object(service.consulterMesures(**{k:message[k] for k,v in donnees.items()}), dict)
            return serialize_object(message), response
        except Exception as e:
            logger.exception("Erreur appel service")
            return message, {'erreur': str(e.args), 'status': 'failed'}




from time import sleep

from django.test import TestCase
from rest_framework.test import APIClient

from enedis.data_models.recherchepoint import *
from enedis.data_models.rechercheservicesouscritmesures import RechercherServicesSouscritsMesures


class TestThrottle(TestCase):

    def setUp(self) -> None:
        sleep(1)

    def test_multiple_call_one_endpoint(self):
        client = APIClient()

        adresse_installation = AdresseInstallation('escalier 8 etage 9 appartement 10', 'batiment E',
                                                   '404 voie introuvable',
                                                   'Faron', '83000', '48414956515')
        criteres = Criteres('12345678901234', '963852', DomaineTension.HTA, 'societe de test',
                            ClientFinalCategorieCode.PRO, True, adresse_installation)
        donnees_recherche_point = RechercherPoint(criteres)
        donnees_entree = {'serveur': 'poste_travail', 'donnees': donnees_recherche_point}

        client.post('/ws/v2.0/recherchepoint/', donnees_entree, format='json')
        reponse = client.post('/ws/v2.0/recherchepoint/', donnees_entree, format='json')

        self.assertEqual(reponse.json()['detail'], 'Request was throttled. Expected available in 1 second.')


    def test_multiple_call_multiple_endpoints(self):
        client = APIClient()
        adresse_installation = AdresseInstallation('escalier 8 etage 9 appartement 10', 'batiment E',
                                                   '404 voie introuvable',
                                                   'Faron', '83000', '48414956515')
        criteres = Criteres('12345678901234', '963852', DomaineTension.HTA, 'societe de test',
                            ClientFinalCategorieCode.PRO, True, adresse_installation)
        donnees_recherche_point = RechercherPoint(criteres)

        donnees_recherche_service = {'serveur': 'poste_travail', 'donnees': RechercherServicesSouscritsMesures({'pointId': '12345678911234', 'contratId':''})}

        client.post('/ws/v2.0/recherchepoint/', {'serveur': 'poste_travail', 'donnees': donnees_recherche_point}, format='json')

        reponse = client.post('/ws/v1.0/rechercheservicessouscritsmesures/', donnees_recherche_service, format='json')

        resultat_attendu = {
            'header': {
                'acquittement': None
            },
            'body': {
                'servicesSouscritsMesures': {
                    'serviceSouscritMesures': [
                        {
                            'serviceSouscritId': 'string', 'pointId': 'string',
                            'serviceSouscritType': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'serviceSouscritLibelle': 'string', 'injection': False, 'soutirage': False,
                            'contratId': 'string',
                            'contratLibelle': 'string', 'etatCode': 'string', 'dateDebut': '2005-09-02',
                            'dateFin': '2019-08-06',
                            'motifFinLibelle': 'string', 'mesuresTypeCode': 'string', 'mesuresPas': 'string',
                            'mesuresCorrigees': False, 'periodiciteTransmission': 'string'
                        }
                    ]
                }
            }
        }
        self.assertEqual(reponse.json(), resultat_attendu)



from django.test import TestCase

from enedis.utils.regex import verification_prm


class TestCheckPrm(TestCase):

    def test_prm(self):
        prm = "01234567891234"
        verification_prm(prm)
        wrong_prm = "01"
        self.assertRaises(ValueError, verification_prm, wrong_prm)



from enedis.utils.regex import verification_prm
from .const import GrandeurPhysique, SensMesure, MesuresPas, CadreAcces, TypeDonnees
import datetime as dt


class DemandeConsulterMesuresDetaillees(dict):
    def __init__(self, point_id: str, mesures_type_code: TypeDonnees, grandeur_physique: GrandeurPhysique,
                 sens: SensMesure,
                 date_debut: dt.date, date_fin: dt.date, mesures_corrigees: bool, cadre_acces: CadreAcces,
                 mesures_pas: MesuresPas = None):
        verification_prm(point_id)
        _kwargs = {
            'initiateurLogin': '',
            'pointId': point_id,
            'mesuresTypeCode': mesures_type_code.value,
            'grandeurPhysique': grandeur_physique.value,
            'sens': sens.value,
            'dateDebut': date_debut,
            'dateFin': date_fin,
            'mesuresCorrigees': mesures_corrigees,
            'cadreAcces': cadre_acces.value
        }

        if mesures_pas is not None:
            _kwargs['mesuresPas'] = mesures_pas.value
        dict.__init__(self, **_kwargs)



from django.test import TestCase
from rest_framework.test import APIClient

from enedis.data_models.consultermesures import DonneesConsulterMesures


class TestViewConsulterMesures(TestCase):
    maxDiff = None

    def test_view_consultermesures(self):
        client = APIClient()
        datas = {'serveur': 'poste_travail', 'donnees': DonneesConsulterMesures('12345678911234', True)}
        reponse = client.post('/ws/v1.1/consultermesures/', datas, format='json')
        resultat_attendu = {
            'header': {
                'acquittement': None
            },
            'body': {
                'seriesMesuresDateesGrilleTurpe': {
                    'serie': [
                        {
                            'grandeurPhysique': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'classeTemporelle': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'calendrier': {
                                'libelle': 'string', 'code': 'string'
                            }, 'unite': 'string',
                            'mesuresDatees': {
                                'mesure': [
                                    {
                                        'valeur': 100, 'dateDebut': '2019-09-25',
                                        'dateFin': '2004-11-30',
                                        'nature': {
                                            'libelle': 'string', 'code': 'string'
                                        },
                                        'declencheur': {
                                            'libelle': 'string', 'code': 'string'
                                        },
                                        'statut': {
                                            'libelle': 'string', 'code': 'string'
                                        }
                                    }
                                ]
                            }
                        }
                    ]
                },
                'seriesMesuresDateesGrilleFrn': {
                    'serie': [
                        {
                            'grandeurPhysique': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'classeTemporelle': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'calendrier': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'unite': 'string',
                            'mesuresDatees': {
                                'mesure': [
                                    {
                                        'valeur': 100, 'dateDebut': '2011-05-04',
                                        'dateFin': '2012-03-25',
                                        'nature': {
                                            'libelle': 'string', 'code': 'string'
                                        },
                                        'declencheur': {
                                            'libelle': 'string', 'code': 'string'
                                        },
                                        'statut': {
                                            'libelle': 'string', 'code': 'string'
                                        }
                                    }
                                ]
                            }
                        }
                    ]
                }
            }
        }
        self.assertDictEqual(reponse.json(), resultat_attendu)






import datetime as dt
import decimal
import json
import os
from pprint import pprint

from django.http import HttpResponse
from rest_framework.response import Response
from rest_framework.views import APIView

from enedis.connection.enedis_connection import EnedisConnection
from enedis.services.commande_M023 import M023HistoriqueMesuresFines, M023MesuresFacturantes, \
    M023InformationsTechniquesEtContractuelles
from enedis.services.commande_acces_donnees_mesures import CommandeAccesDonneesMesures
from enedis.services.commander_arret_service_souscrit_mesures import CommanderArretServiceSouscritMesures
from enedis.services.commander_collecte_publication_mesures import CommanderCollectePublicationMesures
from enedis.services.consulter_donnees_techniques_contractuelles import ConsulterDoneesTechniquesContractuelles
from enedis.services.consulter_mesures import ConsulterMesures
from enedis.services.consulter_mesures_detaillees import ConsulterMesuresDetaillees
from enedis.services.consulter_point import ConsulterPoint
from enedis.services.rechercher_point import RecherchePoint
from enedis.services.recherche_services_souscrits_mesures import RechercheServicesSouscritsMesures


def deserializator(o):
    encoder = json.JSONEncoder()
    if isinstance(o, dt.time) or isinstance(o, dt.datetime) or isinstance(o, dt.date):
        return o.isoformat()
    if isinstance(o, decimal.Decimal):
        return float(o)
    else:
        return encoder.default(o)

class EnedisApiView(APIView):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        enedis_url = os.environ['WS_ENEDIS_URL']
        login = os.environ['WS_ENEDIS_LOGIN']
        contrat_id = os.environ['WS_ENEDIS_CONTRAT_ID']
        self.connection = EnedisConnection(enedis_url, login, contrat_id)

class RecherchePointV2View(EnedisApiView):
    throttle_scope = 'rechercherpointv2'

    def post(self, request, format=None):
        msg, resultat = RecherchePoint.recherche_point(self.connection, request.data['donnees'])
        return Response(resultat)


class RechercherServicesSouscritsMesuresV1View(EnedisApiView):
    throttle_scope = 'rechercherservicessouscritsmesuresv1'

    def post(self, request, format=None):

        pprint(request.data['donnees'])
        msg, resultat = RechercheServicesSouscritsMesures.rechercher_services_souscrit_mesures(self.connection,
            request.data['donnees'])

        return Response(resultat)


class ConsultationPointV4View(EnedisApiView):
    throttle_scope = 'consultationpointv4'

    def post(self, request, format=None):
        msg, resultat = ConsulterPoint.consulter_donnees_point(self.connection, request.data['donnees'])
        return Response(resultat)


class ConsultationMesuresDetailleesV3View(EnedisApiView):
    throttle_scope = 'consultationmesuresdetailleesv3'

    def post(self, request, format=None):
        msg, resultat = ConsulterMesuresDetaillees.consulter_mesures_detaillees(self.connection, request.data['donnees'])
        return Response(resultat)


class ConsultationMesuresV1View(EnedisApiView):
    throttle_scope = 'consultationmesuresv1'

    def post(self, request, format=None):
        msg, resultat = ConsulterMesures.consulter_mesures(self.connection, request.data['donnees'])
        return Response(resultat)


class ConsultationDonneesTechniquesContractuellesV1View(EnedisApiView):
    throttle_scope = 'consultationdonneestechniquescontractuellesv1'

    def post(self, request, format=None):
        msg, resultat = ConsulterDoneesTechniquesContractuelles.consultation_donnees_techniques_contractuelles(
            self.connection, request.data['donnees'])
        return HttpResponse(json.dumps(resultat, default=deserializator).encode(), content_type='application/json')


class CommandeCollectePublicationMesuresV3View(EnedisApiView):
    throttle_scope = 'commandecollectepublicationmesuresv3'

    def post(self, request, format=None):
        msg, resultat = CommanderCollectePublicationMesures.commander_collecte_publication(self.connection, request.data['donnees'])
        return Response(resultat)


class CommandeArretServicesSouscritsMesuresV1View(EnedisApiView):
    throttle_scope = 'commandearretservicessouscritsmesuresv1'

    def post(self, request, format=None):
        msg, resultat = CommanderArretServiceSouscritMesures.commander_arret_service_souscrit_mesures(
            self.connection, request.data['donnees'])
        return Response(resultat)


class CommandeAccesDonneesMesuresV1View(EnedisApiView):
    throttle_scope = 'commandeaccesservicessouscritsv1'

    def post(self, request, format=None):
        msg, resultat = CommandeAccesDonneesMesures.commande_acces_donnees_mesures(self.connection, request.data['donnees'])
        return Response(resultat)

class CommandeHistoriqueDonneesMesuresFinesV1View(EnedisApiView):
    throttle_scope = 'commandem023historiquedonneesmesuresfinesv1'

    def post(self, request, format=None):
        msg, resultat = M023HistoriqueMesuresFines.m023(self.connection, request.data['donnees'])
        return Response(resultat)

class CommandeInformationsTechniquesEtContractuellesV1View(EnedisApiView):
    throttle_scope = 'commandem023informationstechniquesetcontractuellesv1'

    def post(self, request, format=None):
        msg, resultat = M023InformationsTechniquesEtContractuelles.m023(self.connection, request.data['donnees'])
        return Response(resultat)

class CommandeMesuresFacturantesV1View(EnedisApiView):
    throttle_scope = 'commandem023mesuresfacturantesv1'

    def post(self, request, format=None):
        msg, resultat = M023MesuresFacturantes.m023(self.connection, request.data['donnees'])
        return Response(resultat)





class DonneesConsulterMesures(dict):
    def __init__(self, point_id: str, autorisation_client=False):
        _kwargs = {'pointId': point_id, 'loginDemandeur': '', 'contratId': '', 'autorisationClient': autorisation_client}
        dict.__init__(self, **_kwargs)



import datetime as dt
from django.test import TestCase
from rest_framework.test import APIClient

from enedis.data_models.commandercollectepublicationmesures import AccesMesures, DonneesGeneralesDemandeCollecte, \
    DemandeCollectePublicationMesure
from enedis.data_models.common import PersonneMorale, DeclarationAccordClient
from enedis.data_models.const import *


class TestViewCommanderCollectePublicationMesures(TestCase):
    maxDiff = None

    def test_view_commandercollectepublicationmesures(self):
        client = APIClient()
        personne_morale = PersonneMorale('personnemoraleclient')
        declaration_accord_client = DeclarationAccordClient(personne_morale, accord=True)
        acces_mesures = AccesMesures(dt.date.today(), dt.date.today(), declaration_accord_client, TypeDonneesV1.INDEX,
                                     False,
                                     True, PasMesures.QUOTIDIEN, True,
                                     PeriodiciteTransmission.QUOTIDIENNE, False)
        donnees_generales = DonneesGeneralesDemandeCollecte('12345678911234')
        donnees = DemandeCollectePublicationMesure(donnees_generales, acces_mesures)
        entrees = {'serveur': 'poste_travail', 'donnees': donnees}
        reponse = client.post('/ws/v3.0/commandercollectepublicationmesures/', entrees, format='json')
        resultat_attendu = {'header': {
            'acquittement': None
        },
            'body': {
                'affaireId': 'string',
                'prestations': {
                    'prestation': [
                        {
                            'rang': 100,
                            'fiche': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'option': {
                                'libelle': 'string', 'code': 'string'
                            }
                        }
                    ]
                }
                , 'serviceSouscritId': 'string'
            }
        }
        self.assertEqual(reponse.json(), resultat_attendu, "echec du test de connexion  avec imposter")



from enedis.services.rechercher_point import RecherchePoint
import datetime as dt


def run():
    print(f'message envoyé à {dt.datetime.now()}')

    reponse = RecherchePoint.recherche_point(Criteres('12345678911234'))

    print(f'message reçu à {dt.datetime.now()}: \n {reponse}')



import datetime as dt
import enum
import json
import os
from json import JSONEncoder

import requests as r

from enedis.data_models.commandeaccesdonneesmesures import DemandeAccesDonneesMesures
from enedis.data_models.commanderarretservicesouscritmesures import DemandeArretServiceSouscritMesures
from enedis.data_models.commandercollectepublicationmesures import DemandeCollectePublicationMesure
from enedis.data_models.consulterdonneestechniquescontractuelles import ConsulterDonneesTechniquesContractuelles

from enedis.data_models.consultermesures import DonneesConsulterMesures
from enedis.data_models.consultermesuresdetaillees import \
    DemandeConsulterMesuresDetaillees
from enedis.data_models.recherchepoint import Criteres

__version__ = '1.0_dev'


def serializator(o):
    encoder = json.JSONEncoder()
    if isinstance(o, dt.time) or isinstance(o, dt.datetime) or isinstance(o, dt.date):
        return o.isoformat()
    if issubclass(o, enum.Enum):
        return o.value
    else:
        return encoder.default(o)


class ClientProxyEnedis:
    MACHINE = os.getenv('PROJECT_NAME', '')
    WEBSERVICES_ENEDIS_HOST = os.environ['WEBSERVICES_ENEDIS_HOST']
    WEBSERVICES_ENEDIS_ROOT_PATH = os.environ['WEBSERVICES_ENEDIS_ROOT_PATH']

    def _envoi(self, donnees, _adresse=None, serializer=None):
        donnees_complettes = {'serveur': self.MACHINE, 'donnees': {**donnees}}
        rep = r.post(_adresse, json=json.dumps(donnees_complettes, default=serializator or JSONEncoder)).json()
        return rep

    def commander_access_donnees_mesures(self, donnees: DemandeAccesDonneesMesures, version='v1.0'):
        adresse = f'{self.WEBSERVICES_ENEDIS_HOST}{self.WEBSERVICES_ENEDIS_ROOT_PATH}{version}/commandeaccesdonneesmesures'
        resultat = self._envoi(donnees, _adresse=adresse, serializer=serializator)
        return resultat

    def commander_arret_services_souscrits_mesures(self, donnees: DemandeArretServiceSouscritMesures, version='v1.0'):
        adresse = f'/{version}/commanderarretservicessouscritmesures'

        resultat = self._envoi(donnees, _adresse=adresse)
        return resultat

    def commander_collecte_publication_mesures(self, donnees: DemandeCollectePublicationMesure,
                                               version='v3.0'):
        adresse = f'/{version}/commandercollectepublicationmesures'
        resultat = self._envoi(donnees, _adresse=adresse)
        return resultat

    def consulter_donnees_techniques_contractuelles(self, donnees: ConsulterDonneesTechniquesContractuelles, version='v1.0'):
        adresse = f'/{version}/consulterdonneestechniquescontractuelles'
        resultat = self._envoi(donnees, _adresse=adresse)
        return resultat

    def consulter_mesures(self, donnees: DonneesConsulterMesures, version='v1.0'):
        adresse = f'/{version}/consultermesures'
        resultat = self._envoi(donnees, _adresse=adresse)
        return resultat

    def consulter_mesures_detaillees(self, donnees: DemandeConsulterMesuresDetaillees, version='v3.0'):
        adresse = f'/{version}/consultermesuresdetaillees'
        resultat = self._envoi(donnees, _adresse=adresse)
        return resultat

    def rechercher_point(self, donnees: Criteres, version='v2.0'):
        adresse = f'/{version}/recherchepoint'
        resultat = self._envoi(donnees, _adresse=adresse)
        return resultat

    def recherche_services_souscrits_mesures(self, point_id: str, version='v1.0'):
        adresse = f'/{version}/rechercheservicessouscritsmesures'
        donnees = {'pointId': point_id}
        resultat = self._envoi(donnees, _adresse=adresse)
        return resultat



from enedis.utils.regex import verification_prm


class ArretServiceSouscrit(dict):
    def __init__(self, service_souscrit_id: str):
        _kwargs = {'serviceSouscritId': service_souscrit_id}
        dict.__init__(self, **_kwargs)


class DonneesGeneralesArretService(dict):
    def __init__(self, point_id: str, ref_frn=None):
        verification_prm(point_id)
        _kwargs = {'pointId': point_id, 'initiateurLogin': '', 'objetCode': 'ASS', 'contratId': ''}
        if ref_frn is not None:
            _kwargs['refFrn'] = ref_frn
        dict.__init__(self, **_kwargs)


class DemandeArretServiceSouscritMesures(dict):

    def __init__(self, donnees_generales: DonneesGeneralesArretService, arret_service_souscrit: ArretServiceSouscrit):
        dict.__init__(self, donneesGenerales=donnees_generales, arretServiceSouscrit=arret_service_souscrit)

# <xs:complexType name="CommanderArretServiceSouscritMesuresType">
#       <xs:sequence>
#          <xs:element name="demande" type="sc:DemandeType"/>
#       </xs:sequence>
#    </xs:complexType>

#    <xs:complexType name="DemandeType">
#       <xs:sequence>
#          <xs:element name="donneesGenerales" type="sc:DonneesGeneralesType"/>
#          <xs:element name="arretServiceSouscrit" type="sc:ArretServiceSouscritType"/>
#       </xs:sequence>
#    </xs:complexType>

#   <xs:complexType name="DonneesGeneralesType">
#       <xs:sequence>
#          <xs:element name="refFrn" type="ds:Chaine255Type" minOccurs="0"/>
#          <xs:element name="objetCode" type="ds:DemandeObjetCodeType"/>
#          <xs:element name="pointId" type="ds:PointIdType"/>
#          <xs:element name="initiateurLogin" type="ds:UtilisateurLoginType"/>
#          <xs:element name="contratId" type="ds:ContratIdType"/>
#       </xs:sequence>
#    </xs:complexType>

#    <xs:complexType name="ArretServiceSouscritType">
#       <xs:sequence>
#          <xs:element name="serviceSouscritId" type="ds:Chaine15Type"/>
#       </xs:sequence>
#    </xs:complexType>


import datetime as dt
from unittest.mock import patch

import zeep
from zeep.helpers import serialize_object

from enedis.data_models.consultermesures import DonneesConsulterMesures
from enedis.services.consulter_mesures import ConsulterMesures
from proxy.tests.services.ServicesTestCase import raise_transport_errror, ServicesTestCase


class TestConsulterMesures(ServicesTestCase):
    maxDiff = None

    def test_creation_message_correct(self):
        donnees = DonneesConsulterMesures('12345678911234', autorisation_client=True)
        msg, _ = ConsulterMesures.consulter_mesures(self.connection, donnees, send=False)
        expected_msg = {'pointId': '12345678911234', 'loginDemandeur': 'test@test.fr', 'contratId': '1111111',
                            'contratConcluNouveauClientSurSite': None, 'autorisationClient': True}
        self.assertDictEqual(msg, expected_msg)

    def test_connexion(self):
        donnees = DonneesConsulterMesures('12345678911234', True)
        _, response = ConsulterMesures.consulter_mesures(self.connection, donnees)
        expected_response = {
            'header': {
                'acquittement': None
            },
            'body': {
                'seriesMesuresDateesGrilleTurpe': {
                    'serie': [
                        {
                            'grandeurPhysique': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'classeTemporelle': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'calendrier': {
                                'libelle': 'string', 'code': 'string'
                            }, 'unite': 'string',
                            'mesuresDatees': {
                                'mesure': [
                                    {
                                        'valeur': 100, 'dateDebut': dt.date(2019, 9, 25), 'dateFin': dt.date(2004, 11, 30),
                                        'nature': {
                                            'libelle': 'string', 'code': 'string'
                                        },
                                        'declencheur': {
                                            'libelle': 'string', 'code': 'string'
                                        },
                                        'statut': {
                                            'libelle': 'string', 'code': 'string'
                                        }
                                    }
                                ]
                            }
                        }
                    ]
                },
                'seriesMesuresDateesGrilleFrn': {
                    'serie': [
                        {
                            'grandeurPhysique': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'classeTemporelle': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'calendrier': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'unite': 'string',
                            'mesuresDatees': {
                                'mesure': [
                                    {
                                        'valeur': 100, 'dateDebut': dt.date(2011, 5, 4), 'dateFin': dt.date(2012, 3, 25),
                                        'nature': {
                                            'libelle': 'string', 'code': 'string'
                                        },
                                        'declencheur': {
                                            'libelle': 'string', 'code': 'string'
                                        },
                                        'statut': {
                                            'libelle': 'string', 'code': 'string'
                                        }
                                    }
                                ]
                            }
                        }
                    ]
                }
            }
        }
        self.assertDictEqual(response, expected_response)

    @patch.object(zeep.helpers, 'serialize_object', raise_transport_errror)
    def test_connexion_echoue(self):
        donnees = DonneesConsulterMesures('12345678911234', True)
        _, response = ConsulterMesures.consulter_mesures(self.connection, donnees)
        self.assertEqual(response, {'erreur': "('une erreur inconnue est survenue',)", 'status': 'failed'})




from unittest.mock import patch

import zeep
from zeep.helpers import serialize_object

from enedis.data_models.recherchepoint import *
from enedis.services.rechercher_point import RecherchePoint
from proxy.tests.services.ServicesTestCase import raise_transport_errror, ServicesTestCase


class TestRecherchePoint(ServicesTestCase):
    maxDiff = None

    def test_creation_message(self):
        adresse_installation = AdresseInstallation('escalier 8 etage 9 appartement 10', 'batiment E', '404 voie introuvable',
                                                   'Faron', '83000', '48414956515')
        criteres = Criteres('12345678901234', '963852', DomaineTension.HTA, 'societe de test', ClientFinalCategorieCode.PRO, True, adresse_installation)
        donnees = RechercherPoint(criteres)
        msg, _ = RecherchePoint.recherche_point(self.connection, donnees, send=False)
        expected_msg = {'criteres': {
            'adresseInstallation': {
                'escalierEtEtageEtAppartement': 'escalier 8 etage 9 appartement 10', 'batiment': 'batiment E',
                'numeroEtNomVoie': '404 voie introuvable', 'lieuDit': 'Faron', 'codePostal': '83000',
                'codeInseeCommune': '48414956515'
            },
            'numSiret': '12345678901234', 'matriculeOuNumeroSerie': '963852', 'domaineTensionAlimentationCode': 'HTA',
            'nomClientFinalOuDenominationSociale': 'societe de test', 'categorieClientFinalCode': 'PRO',
            'rechercheHorsPerimetre': True
        },
            'loginUtilisateur': 'test@test.fr'}

        self.assertDictEqual(msg, expected_msg)

    def test_connexion(self):
        adresse_installation = AdresseInstallation('escalier 8 etage 9 appartement 10', 'batiment E',
                                                   '404 voie introuvable',
                                                   'Faron', '83000', '48414956515')
        criteres = Criteres('12345678901234', '963852', DomaineTension.HTA, 'societe de test',
                            ClientFinalCategorieCode.PRO, True, adresse_installation)
        donnees = RechercherPoint(criteres)

        _, response = RecherchePoint.recherche_point(self.connection, donnees)

        if 'code' in response['body']['points']['point'][0]['etatContractuel']:
            response['body']['points']['point'][0]['etatContractuel']['code'] = 'SERVC'

        expected_response = {
            'header': {
                'acquittement': None
            },
            'body': {
                'points': {
                    'point': [
                        {
                            'adresseInstallationNormalisee': {
                                'ligne1': 'string', 'ligne2': 'string', 'ligne3': 'string', 'ligne4': 'string',
                                'ligne5': 'string', 'ligne6': 'string', 'ligne7': 'string'}, 'matricule': ['string'],
                            'numeroSerie': ['string'],
                            'typeComptage': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'etatContractuel': {
                                'libelle': 'string', 'code': 'SERVC'
                            },
                            'nomClientFinalOuDenominationSociale': 'string', 'id': 'string'
                        }
                    ]
                }
            }
        }
        self.assertDictEqual(response, expected_response)

    @patch.object(zeep.helpers, 'serialize_object', raise_transport_errror)
    def test_connexion_echoue(self):
        adresse_installation = AdresseInstallation('escalier 8 etage 9 appartement 10', 'batiment E',
                                                   '404 voie introuvable',
                                                   'Faron', '83000', '48414956515')
        criteres = Criteres('12345678901234', '963852', DomaineTension.HTA, 'societe de test',
                            ClientFinalCategorieCode.PRO, True, adresse_installation)
        donnees = RechercherPoint(criteres)

        _, response = RecherchePoint.recherche_point(self.connection, donnees)
        self.assertEqual(response, {'erreur': "('une erreur inconnue est survenue',)", 'status': 'failed'})






from django.test import TestCase
from rest_framework.test import APIClient

from enedis.data_models.consultermesures import DonneesConsulterMesures
from proxy.models import TraceActivite


class TestTrace(TestCase):

    def test_log_base(self):
        client = APIClient()
        mauvaise_datas = {'serveur': 'pas le dernier', 'donnees': DonneesConsulterMesures('12345678911234', True)}
        client.post('/v1.1/consultermesures/', mauvaise_datas, format='json')
        client.post('/v1.1/consultermesures/', mauvaise_datas, format='json')
        client.post('/v1.1/consultermesures/', mauvaise_datas, format='json')
        client.post('/v1.1/consultermesures/', mauvaise_datas, format='json')

        datas = {'serveur': 'poste_test_base_de_donnees', 'donnees': DonneesConsulterMesures('95376291856439', True)}
        client.post('/v1.1/consultermesures/', datas, format='json')
        derniere_entree = TraceActivite.objects.latest('horodate')
        self.assertEqual(derniere_entree.RAE_concerne, datas['donnees']['pointId'])
        self.assertEqual(derniere_entree.demandeur, datas['serveur'])



from __future__ import annotations

import datetime as dt

from enedis.utils.regex import verification_prm
from .common import DeclarationAccordClient
from .const import TypeDonneesV1, PeriodiciteTransmission, PasMesures


class AccesMesures(dict):
    def __init__(self, date_debut: dt.date, date_fin: dt.date, declaration_accord_client: DeclarationAccordClient,
                 mesure_type: TypeDonneesV1, soutirage: bool, injection: bool, mesures_pas: PasMesures,
                 transmission_recurrente: bool, periodicite_transmission: PeriodiciteTransmission = None,
                 mesures_corrigees: bool = None):
        _kwargs = {'dateDebut': date_debut, 'dateFin': date_fin, 'declarationAccordClient': declaration_accord_client,
                   'mesuresTypeCode': mesure_type.value, 'soutirage': soutirage, 'injection': injection,
                   'mesuresPas': mesures_pas.value,
                   'transmissionRecurrente': transmission_recurrente}
        if mesures_corrigees is not None:
            _kwargs['mesuresCorrigees'] = mesures_corrigees
        if periodicite_transmission is not None:
            _kwargs['periodiciteTransmission'] = periodicite_transmission.value

        dict.__init__(self, **_kwargs)


class DonneesGeneralesDemandeCollecte(dict):
    def __init__(self, point_id: str, objet_code='AME', ref_externe=None):
        verification_prm(point_id)
        _kwargs = {'pointId': point_id, 'objetCode': objet_code, 'initiateurLogin': '', 'contratId': ''}
        if ref_externe is not None:
            _kwargs['refExterne'] = ref_externe
        dict.__init__(self, **_kwargs)


class DemandeCollectePublicationMesure(dict):
    def __init__(self, donnees_generales: DonneesGeneralesDemandeCollecte, acces_mesures: AccesMesures):
        dict.__init__(self, donneesGenerales=donnees_generales, accesMesures=acces_mesures)



"""
URL configuration for djangoProject project.

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/4.2/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.urls import path

from proxy import views

urlpatterns = [
    path('v2.0/recherchepoint/', views.RecherchePointV2View.as_view()),
    path('v1.0/rechercheservicessouscritsmesures/', views.RechercherServicesSouscritsMesuresV1View.as_view()),
    path('v4.0/consulterpoint/', views.ConsultationPointV4View.as_view()),
    path('v3.0/consultermesuresdetaillees/', views.ConsultationMesuresDetailleesV3View.as_view()),
    path('v1.1/consultermesures/', views.ConsultationMesuresV1View.as_view()),
    path('v1.0/consulterdonneestechniquescontractuelles/', views.ConsultationDonneesTechniquesContractuellesV1View.as_view()),
    path('v3.0/commandercollectepublicationmesures/', views.CommandeCollectePublicationMesuresV3View.as_view()),
    path('v1.0/commanderarretservicessouscritmesures/', views.CommandeArretServicesSouscritsMesuresV1View.as_view()),
    path('v1.0/commandeaccesdonneesmesures/', views.CommandeAccesDonneesMesuresV1View.as_view()),

    # M023 service
    path('v1.0/m023/CommandeHistoriqueDonneesMesuresFines/', views.CommandeHistoriqueDonneesMesuresFinesV1View.as_view()),
    path('v1.0/m023/CommandeInformationsTechniquesEtContractuelles/', views.CommandeInformationsTechniquesEtContractuellesV1View.as_view()),
    path('v1.0/m023/CommandeAccesDonneesMesures/', views.CommandeAccesDonneesMesuresV1View.as_view()),
]




from enedis.utils.regex import verification_prm


class ConsulterDonneesTechniquesContractuelles(dict):
    def __init__(self, point_id: str, autorisation_client=False):
        verification_prm(point_id)
        dict.__init__(self, pointId=point_id, autorisationClient=autorisation_client, loginUtilisateur='')



from proxy.models import TraceActivite
import json
import logging



class EnregistreurActivite:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if len(request.body) == 0:
            return self.get_response(request)
        body = json.loads(request.body.decode("utf-8"))
        trace = TraceActivite()
        trace.service_concerne = request.path
        trace.demandeur = body['serveur']
        if 'pointId' in body['donnees']:
            trace.RAE_concerne = body['donnees']['pointId']
        trace.save()
        # print("rapport: ", copie.path, body['serveur'], trace.RAE_concerne)
        # print("on a cette entrée dans la BDD",TraceActivite.objects.get(id=1).service_concerne, TraceActivite.objects.get(id=1).demandeur, TraceActivite.objects.get(id=1).RAE_concerne, TraceActivite.objects.get(id=1).horodate)
        return self.get_response(request)



import logging
from typing import Tuple

from zeep.helpers import serialize_object

from enedis.connection.enedis_connection import EnedisConnection
from enedis.data_models.commandeaccesdonneesmesures import DemandeAccesDonneesMesures
from enedis.utils.regex import verification_prm

logger = logging.getLogger(__name__)

class CommandeAccesDonneesMesures:
    ns = 'http://www.enedis.fr/sge/b2b/commanderaccesdonneesmesures/v1.0'
    tn_binding = 'http://www.enedis.fr/sge/b2b/commandeaccesdonneesmesures/v1.0'
    name = 'CommanderAccesDonneesMesures-V1.0'
    port_binding = 'CommanderAccesDonneesMesuresBinding'
    service_path = 'CommanderAccesDonneesMesures/v1.0'

    @staticmethod
    def _create_xml_message(donnees: DemandeAccesDonneesMesures, client, service):

        xml = client.create_message(service, 'commanderAccesDonneesMesures', demande=donnees)
        return xml

    @classmethod
    def commande_acces_donnees_mesures(cls, connection: EnedisConnection, donnees: DemandeAccesDonneesMesures, send=True) -> Tuple[dict, dict]:
        message, response = None, None
        try:
            service, factory, client = connection.soap_service(cls.name, cls.ns, cls.port_binding, cls.service_path,
                                                           cls.tn_binding)
            verification_prm(donnees['donneesGenerales']['pointId'])

            donnees['donneesGenerales']['initiateurLogin'] = connection.login
            donnees['donneesGenerales']['contrat']['contratId'] = connection.contract_id

            message = factory.CommanderAccesDonneesMesuresType(donnees)

            if send:
                response = serialize_object(service.commanderAccesDonneesMesures(message), dict)
            return serialize_object(message), response
        except Exception as e:
            logger.exception("Erreur appel service")
            return message, {'erreur': str(e.args), 'status': 'failed'}




from unittest.mock import patch

import zeep
from zeep.helpers import serialize_object

from enedis.data_models.commandeaccesdonneesmesures import *
from enedis.data_models.common import *
from enedis.data_models.const import TypeDonneesV1
from enedis.services.commande_acces_donnees_mesures import CommandeAccesDonneesMesures
from proxy.tests.services.ServicesTestCase import ServicesTestCase, raise_transport_errror


class TestCommandeAccesDonneesMesures(ServicesTestCase):
    maxDiff = None

    def test_acreation_message_correct(self):
        personne_morale = PersonneMorale('denominationsociale')
        declaration_accord_client = DeclarationAccordClient(personne_morale, accord=True)
        acces_donnees = AccesDonnees(dt.date(2020, 6, 22),
                                     dt.date(2022, 6, 22),
                                     TypeDonneesV1.CDC,
                                     TypeSite.SOUTIRAGE,
                                     declaration_accord_client)
        contrat = Contrat()
        donnees_generales = DonneesGeneralesAccesDonnees('12345678911234', contrat)
        donnees = DemandeAccesDonneesMesures(donnees_generales, acces_donnees)

        message, _ = CommandeAccesDonneesMesures.commande_acces_donnees_mesures(self.connection, donnees, send=False)
        expected_message = {
            'demande':{
                'donneesGenerales': {
                    'objetCode': 'AME',
                    'pointId': '12345678911234',
                    'initiateurLogin': 'test@test.fr',
                    'contrat': {
                        'contratId': '1111111'
                    }
                },
                'accesDonnees': {
                    'dateDebut': dt.date(2020, 6, 22),
                    'dateFin': dt.date(2022, 6, 22),
                    'typeDonnees': 'CDC',
                    'declarationAccordClient': {
                        'accord': True,
                        'personneMorale': {
                            'denominationSociale': 'denominationsociale'
                        },
                    },
                    'soutirage': True
                }
            }
        }
        self.assertDictEqual(message, expected_message)


    def test_avec_id_malforme(self):
        personne_morale = PersonneMorale('denominationsociale')
        declaration_accord_client = DeclarationAccordClient(personne_morale, accord=True)
        acces_donnees = AccesDonnees(dt.date(2020, 6, 22), dt.date(2022, 6, 22), TypeDonneesV1.CDC,
                                     TypeSite.SOUTIRAGE,
                                     declaration_accord_client)
        contrat = Contrat()

        donnees_generales = DonneesGeneralesAccesDonnees('1234', contrat)
        donnees = DemandeAccesDonneesMesures(donnees_generales, acces_donnees)

        msg, response = CommandeAccesDonneesMesures.commande_acces_donnees_mesures(self.connection, donnees)
        self.assertEqual(response['erreur'], "('le champ point_id doit être une chaine de 14 caractères numériques',)"
                         , "la vérification de la validité de l'id ne fonctionne plus")


    def test_transmission_message(self):
        personne_morale = PersonneMorale('denominationsociale')
        declaration_accord_client = DeclarationAccordClient(personne_morale, accord=True)
        acces_donnees = AccesDonnees(dt.date(2020, 6, 22), dt.date(2022, 6, 22), TypeDonneesV1.CDC,
                                     TypeSite.SOUTIRAGE,
                                     declaration_accord_client)
        contrat = Contrat()
        donnees_generales = DonneesGeneralesAccesDonnees('12345678911234', contrat)
        donnees = DemandeAccesDonneesMesures(donnees_generales, acces_donnees)
        _, response = CommandeAccesDonneesMesures.commande_acces_donnees_mesures(self.connection, donnees)

        expected_response = {
            'header':
                {
                    'acquittement': None
                },
            'body':
                {
                    'affaireId': 'string',
                    'prestations': {
                        'prestation': {
                            'rang': 100,
                            'fiche': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'option': {
                                'libelle': 'string', 'code': 'string'
                            }, 'cas': {
                                'libelle': 'string', 'code': 'string'
                            }
                        }
                    }, 'serviceSouscritId': 'string'
                }
        }

        self.assertDictEqual(response, expected_response, "echec du test de connexion  avec imposter")

    @patch.object(zeep.helpers, 'serialize_object', raise_transport_errror)
    def test_connexion_echoue(self):
        """
        ./manage.py test tests.test_commandeaccesservicessouscrits.TestCommandeAccesServicesSousCrits.test_connexion_echoue
        """
        personne_morale = PersonneMorale('denominationsociale')
        declaration_accord_client = DeclarationAccordClient(personne_morale, accord=True)
        acces_donnees = AccesDonnees(dt.date(2020, 6, 22), dt.date(2022, 6, 22), TypeDonneesV1.CDC,
                                     TypeSite.SOUTIRAGE,
                                     declaration_accord_client)
        contrat = Contrat()
        donnees_generales = DonneesGeneralesAccesDonnees('12345678911234', contrat)
        donnees = DemandeAccesDonneesMesures(donnees_generales, acces_donnees)
        msg, response = CommandeAccesDonneesMesures.commande_acces_donnees_mesures(self.connection, donnees)
        self.assertEqual(response, {'erreur': "('une erreur inconnue est survenue',)", 'status': 'failed'})




# Generated by Django 4.2.2 on 2023-06-29 14:55

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TraceActivite',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('demandeur', models.CharField(max_length=50)),
                ('RAE_concerne', models.CharField(max_length=20)),
                ('service_concerne', models.CharField(max_length=100)),
                ('horodate', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]



from enedis.utils.regex import verification_prm


class Criteres(dict):
    def __init__(self, point_id):
        verification_prm(point_id)
        dict.__init__(self, contratId='', pointId=point_id)

class RechercherServicesSouscritsMesures(dict):
    def __init__(self, criteres: Criteres):
        dict.__init__(self, criteres=criteres, loginUtilisateur='')




from django.test import TestCase
from rest_framework.test import APIClient

from enedis.data_models.consulterdonneespoint import DonneesConsulterPoint


class TestViewConsulterPoint(TestCase):
    maxDiff = None

    def test_view_consulterpoint(self):
        client = APIClient()
        donnees = DonneesConsulterPoint('12345678911234', autorisation_client=True)
        entree = {'serveur': 'poste_travail', 'donnees': donnees}
        reponse = client.post('/ws/v4.0/consulterpoint/', entree, format='json')
        resultat_attendu = {'header': {'acquittement': None}, 'body': {'point': {'donneesGenerales': {
            'etatContractuel': {'libelle': 'string', 'code': 'IMPRO'}, 'adresseInstallation':
                {'escalierEtEtageEtAppartement': 'string', 'batiment': 'string',
                 'numeroEtNomVoie': 'string', 'lieuDit': 'string', 'codePostal': 'string',
                 'commune': {'libelle': 'string', 'code': 'string'}},
            'alimentationComplementaire': False, 'alimentationSecours': False, 'bornePoste':
                True, 'borneFixe': False, 'hebergeurDecomptant': {'libelle': 'string',
                                                                  'code': 'DECO'},
            'regroupementTurpe': {'libelle': 'string', 'code': 'NON'},
            'dateDerniereModificationFormuleTarifaireAcheminement': '2011-12-04',
            'dateDerniereModificationCalendrierFournisseur': '2003-04-20',
            'dateDerniereModificationGroupePeriodeMobile': '2014-12-02',
            'dateDerniereAugmentationPuissanceSouscrite': '2012-07-23',
            'dateDerniereDiminutionPuissanceSouscrite': '2010-07-29',
            'datePremierePoseCompteurLinky': '2018-12-27', 'segment': {
                'libelle': 'string', 'code': 'C2'}, 'numTelephoneDepannage': 'string',
            'niveauOuvertureServices': 'string', 'sensible': True,
            'autoconsommationIndividuelle': {'libelle': 'string', 'code': 'string'},
            'rattachements': {'rattachement': [{'type': {'libelle': 'string',
                                                         'code': 'string'}, 'dateDebut': '2006-03-19',
                                                'dateFin':
                                                    '2008-01-24', 'pointsConfondus': True,
                                                'longueursLiaisons': {
                                                    'longueurLiaisonAerienneHta': {
                                                        'valeur': 1000.0,
                                                        'unite': 'm'},
                                                    'longueurLiaisonSouterraineHta': {'valeur': 1000.0, 'unite': 'km'}},
                                                'pointsRattaches': {'pointRattache': [{
                                                    'role': {'libelle': 'string', 'code': 'string'},
                                                    'puissanceMaxAppelee': {
                                                        'valeur': 1000.0, 'unite': 'kVA'},
                                                    'finalites': {
                                                        'finalite': [{'libelle': 'string', 'code': 'string'}]},
                                                    'id': 'string'}]}}]}},
            'situationAlimentation': {'etatAlimentation': {'libelle': 'string',
                                                           'code': 'NRAC'},
                                      'alimentationPrincipale': {'domaineTension': {'libelle':
                                                                                        'string', 'code': 'BTSUP'},
                                                                 'tensionLivraison': {'libelle': 'string',
                                                                                      'code': 'string'},
                                                                 'nbFilsBranchement': 100,
                                                                 'modeApresCompteur': {'libelle':
                                                                                           'string', 'code': 'string'},
                                                                 'longueursLiaisons': {'longueurLiaisonAerienne': {
                                                                     'valeur': 1000.0,
                                                                     'unite': 'km'},
                                                                     'longueurLiaisonSouterraine': {
                                                                         'valeur': 1000.0,
                                                                         'unite': 'm'}},
                                                                 'puissanceRaccordementSoutirage': {'valeur': 1000.0, 'unite': 'kVA'},
                                                                 'puissanceLimiteSoutirage': {'valeur':
                                                                                                  1000.0,
                                                                                              'unite': 'kWc'},
                                                                 'zoneQualiteDesserte': {
                                                                     'libelle': 'string', 'code': 'string'},
                                                                 'installationClient': {
                                                                     'nbMoyensProductionAutonome': 100,
                                                                     'puissanceTotaleProductionAutonome': {
                                                                         'valeur': 1000.0,
                                                                         'unite': 'kVA'},
                                                                     'modeCouplageProductionAutonome': {
                                                                         'libelle': 'string', 'code': 'string'},
                                                                     'dispositifParticulierLimitationPerturbations': 'string'}},
                                      'alimentationsComplementaires': {'alimentationComplementaire': [{
                                          'domaineTension': {'libelle': 'string', 'code': 'BTSUP'},
                                          'tensionLivraison': {
                                              'libelle': 'string', 'code': 'string'}, 'longueursLiaisons': {
                                              'longueurLiaisonAerienne': {'valeur': 1000.0,
                                                                          'unite': 'm'},
                                              'longueurLiaisonSouterraine': {'valeur': 1000.0, 'unite': 'm'}}, 'nbCellules': 100,
                                          'puissanceRaccordementSoutirage': {'valeur': 1000.0,
                                                                             'unite': 'kVAR'}}]},
                                      'alimentationsSecours': {'alimentationSecours': [{
                                          'domaineTension': {'libelle': 'string', 'code': 'HTB'}, 'tensionLivraison': {
                                              'libelle': 'string', 'code': 'string'}, 'longueursLiaisons': {
                                              'longueurLiaisonAerienne': {'valeur': 1000.0,
                                                                          'unite': 'm'},
                                              'longueurLiaisonSouterraine': {'valeur': 1000.0, 'unite': 'm'}},
                                          'transformateurDifferentPrincipal':
                                              True, 'puissanceReserveeHt': {'valeur': 1000.0,
                                                                            'unite': 'kWc'},
                                          'puissanceReserveeBt': {'valeur': 1000.0, 'unite': 'kW'}, 'modeBasculePrincipaleSecours': {
                                              'libelle': 'string', 'code': 'string'}, 'prorataPuissance': 1000.0, 'nbCellules': 100,
                                          'alimentationGarantie': True}]},
                                      'coupure': {'dateDebut': '2010-08-31', 'motif': {'libelle':
                                                                                                   'string',
                                                                                               'code': 'string'},
                                                  'localisation': {'libelle': 'string',
                                                                   'code': 'string'}},
                                      'limitation': {'puissanceLimitee': {'valeur': 1000.0, 'unite': 'kVAR'},
                                          'typeLimiteur': {'libelle': 'string',
                                                           'code': 'string'}}},
            'situationComptage': {'dispositifComptage': {'typeComptage':
                                                             {'libelle': 'string', 'code': 'string'},
                                                         'compteurs': {'compteur': [{
                                                             'numeroSerie': 'string',
                                                             'localisation': {'libelle': 'string', 'code': 'string'},
                                                             'accessibilite': True,
                                                             'regimePropriete': {'libelle': 'string',
                                                                                 'code': 'string'},
                                                             'modeleCompteur': {'typeAppareil': 'string', 'sousType': {
                                                                 'libelle': 'string', 'code': 'string'},
                                                                                'anneeFabrication': [2007, None],
                                                                                'nbRoues': 100, 'nbCadrans': 100},
                                                             'matricule': 'string', 'tensionCompteur': {
                                                                 'libelle': 'string', 'code': 'string'},
                                                             'puissanceMax': {'valeur': 1000.0, 'unite': 'kVA'},
                                                             'nombreFils': 100, 'intensiteNominale':
                                                                 {'libelle': 'string', 'code': 'string'},
                                                             'ticActivee': False, 'ticStandard':
                                                                 False, 'ticActivable': True,
                                                             'plagesHeuresCreuses': 'string',
                                                             'periodeDeploiementLinky': 'string',
                                                             'parametresTeleAcces': {'numeroTelephone':
                                                                                         'string',
                                                                                     'numeroVoieAiguillage': 100,
                                                                                     'cle': 'string',
                                                                                     'etatLigneTelephonique':
                                                                                         'string',
                                                                                     'fenetreEcouteClient': {
                                                                                         'heureDebut': '04:57:17',
                                                                                         'duree': {'valeur': 1000.0,
                                                                                                   'unite': 's'}}},
                                                             'programmationHoraire': {
                                                                 'programmationPosteHoraire': [{'libelle': 'string',
                                                                                                'periodesHoraires': 'string',
                                                                                                'code': 'string'}]},
                                                             'coefficientLecture': 1000.0,
                                                             'eligiblePeriodeMobile': False}]}, 'grilleFrn': {
                    'calendrier': {'libelle': 'string', 'code': 'string'}}, 'presenceBtr': True,
                                                         'disjoncteur': {
                                                             'localisation': {'libelle': 'string', 'code': 'string'},
                                                             'accessibilite': False,
                                                             'regimePropriete': {'libelle': 'string',
                                                                                 'code': 'string'},
                                                             'nature': {'libelle': 'string', 'code': 'string'},
                                                             'intensiteReglage': {
                                                                 'valeur': 1000.0,
                                                                 'unite': 'A'},
                                                             'calibre': {'libelle': 'string', 'code': 'string'},
                                                             'nbPoles': 100}, 'relais': {
                    'regimePropriete': {'libelle': 'string', 'code': 'string'}, 'nature': {'libelle':
                                                                                               'string',
                                                                                           'code': 'string'},
                    'typeCommande': {'libelle': 'string',
                                     'code': 'string'}, 'plageHeuresCreuses': 'string'}, 'transformateurCourant': {
                    'localisation': {'libelle': 'string', 'code': 'string'}, 'accessibilite': False,
                    'regimePropriete': {'libelle': 'string', 'code': 'string'}, 'calibre': {
                        'libelle': 'string', 'code': 'string'}, 'couplage': {'libelle': 'string',
                                                                             'code': 'string'},
                    'classePrecision': {'libelle': 'string', 'code': 'string'},
                    'position': {'libelle': 'string', 'code': 'string'}}, 'transformateurTension': {
                    'localisation': {'libelle': 'string', 'code': 'string'}, 'accessibilite': False,
                    'regimePropriete': {'libelle': 'string', 'code': 'string'}, 'calibre': {
                        'libelle': 'string', 'code': 'string'}, 'couplage': {'libelle': 'string',
                                                                             'code': 'string'},
                    'classePrecision': {'libelle': 'string', 'code': 'string'}},
                                                         'facteurCorrectifPertesJoule': 1000.0,
                                                         'facteurCorrectifPertesFer': 1000.0,
                                                         'facteurCorrectifPertesReactives': 1000.0,
                                                         'particularite': {'libelle': 'string', 'code': 'string'},
                                                         'tensionComptage': {
                                                             'libelle': 'string', 'code': 'string'}},
                                  'caracteristiquesReleve': {
                                      'modeTraitement': {'libelle': 'string', 'code': 'string'}, 'periodicite': {
                                          'libelle': 'string', 'code': 'string'}, 'dureeCycle': {'valeur': 1000.0,
                                                                                                 'unite': 'min'}, 'plageReleve': {'libelle': 'string',
                                                                                                    'code': 'string'}},
                                  'modeReleve': {'libelle': 'string', 'code': 'string'},
                                  'mediaReleve': {'libelle': 'string', 'code': 'string'},
                                  'futuresPlagesHeuresCreuses': {'libelle': 'string', 'code': 'string'},
                                  'futuresProgrammationsHoraires': {'formuleTarifaireAcheminement': [{'libelle':
                                                                                                          'string',
                                                                                                      'programmationHoraire': {
                                                                                                          'programmationPosteHoraire': [
                                                                                                              {
                                                                                                                  'libelle':
                                                                                                                      'string',
                                                                                                                  'periodesHoraires': 'string',
                                                                                                                  'code': 'string'}]},
                                                                                                      'code': 'string'}]},
                                  'teleoperable': True, 'enregistrementCourbeDeCharge': False},
            'situationContractuelle': {'dateDebut': '2011-04-14', 'dateFin':
                '2003-08-19', 'contratId': 'string', 'clientFinal': {'categorie': {
                'libelle': 'string', 'code': 'PRO'}, 'typeResidence': {'libelle': 'string',
                                                                       'code': 'SEC'}, 'refFournisseur': 'string',
                'personnePhysique': {'civilite':
                                         'string', 'nom': 'string', 'prenom': 'string'}, 'personneMorale': None,
                'adressePostale': {'ligne1': 'string', 'ligne2': 'string', 'ligne3': 'string',
                                   'ligne4': 'string', 'ligne5': 'string', 'ligne6': 'string', 'ligne7': 'string'},
                'coordonneesContact': {'numTelephone1': 'string', 'numTelephone2': 'string',
                                       'numFax': 'string', 'adresseEmail': 'string'}}, 'interlocuteurClient': {
                'personnePhysique': {'civilite': 'string', 'nom': 'string', 'prenom': 'string'},
                'personneMorale': None, 'adressePostale': {'ligne1': 'string', 'ligne2':
                    'string', 'ligne3': 'string', 'ligne4': 'string', 'ligne5': 'string',
                                                           'ligne6': 'string', 'ligne7': 'string'},
                'coordonneesContact': {'numTelephone1':
                                           'string', 'numTelephone2': 'string', 'numFax': 'string', 'adresseEmail':
                                           'string'}},
                                       'interlocuteurTechnique': {'personnePhysique': {'civilite': 'string',
                                                                                       'nom': 'string',
                                                                                       'prenom': 'string'},
                                                                  'personneMorale': None, 'adressePostale': {
                                               'ligne1': 'string', 'ligne2': 'string', 'ligne3': 'string',
                                               'ligne4': 'string',
                                               'ligne5': 'string', 'ligne6': 'string', 'ligne7': 'string'},
                                                                  'coordonneesContact': {'numTelephone1': 'string',
                                                                                         'numTelephone2': 'string',
                                                                                         'numFax': 'string',
                                                                                         'adresseEmail': 'string'}},
                                       'structureTarifaire': {
                                           'formuleTarifaireAcheminement': {'libelle': 'string', 'code': 'string'},
                                           'longueUtilisation': {'contexte': {'libelle': 'string', 'code': 'string'},
                                                                 'forfait': {'valeur': 1000.0,
                                                                             'unite': 'jour'}},
                                           'codeTarifAcheminement': 'string',
                                           'puissanceSouscriteMax': {'valeur': 1000.0, 'unite': 'kW'},
                                           'denivelePuissances': {
                                               'classesTemporelles': {
                                                   'classeTemporelle': [{'libelle': 'string', 'puissance': {
                                                       'valeur': 1000.0, 'unite': 'kVAR'},
                                                                         'code': 'string'}]}},
                                           'calendrierFrn': {'libelle': 'string', 'code': 'string'},
                                           'groupePeriodeMobile': {'libelle': 'string', 'code': 'string'}},
                                       'siContractuel':
                                           {'libelle': 'string', 'code': 'string'}},
            'optionsContractuellesSouscrites': {
                'engagementQualiteFourniture': [{'dateSouscription': '2011-07-22',
                                                 'dateResiliation': '2004-03-03',
                                                 'dureeCreux': {'valeur': 1000.0, 'unite': 'jour'},
                                                 'periodicite': {'libelle': 'string',
                                                                 'code': 'string'}, 'nbCreux': 100,
                                                 'profondeurCreux': 1000.0,
                                                 'dateReferenceCreux': '2017-08-12',
                                                 'id': 201}],
                'telecommandeInterrupteurs': [{'dateSouscription': '2009-07-12', 'dateResiliation': '2018-08-02', 'nbDirections':
                    100}], 'protectionsChantier': [{'dateSouscription': '2001-10-24',
                                                    'dateResiliation': '2001-02-20', 'nbPortees': 100}],
                'courbesCharge': [{'dateSouscription': '2017-09-14',
                                   'dateResiliation': '2008-04-17', 'transmission': True,
                                   'utilisationRecoflux': False, 'periodicite': {'valeur': 1000.0,
                                                                                 'unite': 'min'},
                                   'pasCourbeCharge': {'valeur': 1000.0, 'unite': 'jour'}}], 'engagementContinuiteFourniture': [{
                    'dateSouscription': '2007-05-31', 'dateResiliation': '2012-03-23',
                    'type': {'libelle': 'string', 'code': 'string'}, 'periodicite': {
                        'libelle': 'string', 'code': 'string'}, 'typesCoupures': {'libelle': 'string',
                                                                                  'code': 'string'},
                    'nbCoupuresBreves': 100, 'nbCoupuresLongues': 100,
                    'nbTotalCoupures': 100, 'dateReferenceCoupures': '2014-12-18',
                    'id': 201}], 'periodeObservation': [{'dateSouscription': '2005-09-05',
                                                         'dateResiliation': '2010-02-23',
                                                         'duree': {'valeur': 1000.0, 'unite': 'semaine'}}],
                'calendrierFournisseur': [{
                    'dateSouscription': '2013-10-31', 'dateResiliation':
                        '2018-08-10', 'calendrier': {'libelle': 'string',
                                                             'code': 'string'}, 'profilable': False,
                    'periodeMobileAutorisee': False}]},
            'modificationsContractuellesEnCours': {'modificationContractuelleEnCours': [{
                'prestations': {'prestation': [{'fiche': {'libelle': 'string', 'code': 'string'},
                                                'option': {'libelle': 'string', 'code': 'string'},
                                                'cas': {'libelle': 'string',
                                                        'code': 'string'}, 'rang': 100,
                                                'dateEffetReelle': '2019-03-30',
                                                'dateEffetPrevue': '2001-09-30'}]}, 'demande': {'contratId':
                                                                                                            'string',
                                                                                                        'dateEffetSouhaitee': '2003-07-15',
                                                                                                        'clientFinal': {
                                                                                                            'categorie': {
                                                                                                                'libelle': 'string',
                                                                                                                'code': 'RES'},
                                                                                                            'typeResidence': {
                                                                                                                'libelle':
                                                                                                                    'string',
                                                                                                                'code': 'PRP'},
                                                                                                            'refFournisseur': 'string',
                                                                                                            'personnePhysique': {
                                                                                                                'civilite': 'string',
                                                                                                                'nom': 'string',
                                                                                                                'prenom': 'string'},
                                                                                                            'personneMorale':
                                                                                                                None,
                                                                                                            'adressePostale': {
                                                                                                                'ligne1': 'string',
                                                                                                                'ligne2': 'string',
                                                                                                                'ligne3': 'string',
                                                                                                                'ligne4': 'string',
                                                                                                                'ligne5': 'string',
                                                                                                                'ligne6': 'string',
                                                                                                                'ligne7': 'string'},
                                                                                                            'coordonneesContact': {
                                                                                                                'numTelephone1': 'string',
                                                                                                                'numTelephone2': 'string',
                                                                                                                'numFax': 'string',
                                                                                                                'adresseEmail': 'string'}},
                                                                                                        'structureTarifaire': {
                                                                                                            'formuleTarifaireAcheminement': {
                                                                                                                'libelle': 'string',
                                                                                                                'code': 'string'},
                                                                                                            'longueUtilisation': {
                                                                                                                'contexte': {
                                                                                                                    'libelle': 'string',
                                                                                                                    'code': 'string'},
                                                                                                                'forfait': {
                                                                                                                    'valeur': 1000.0,
                                                                                                                    'unite': 'ms'}},
                                                                                                            'codeTarifAcheminement': 'string',
                                                                                                            'puissanceSouscriteMax': {
                                                                                                                'valeur': 1000.0,
                                                                                                                'unite': 'kW'},
                                                                                                            'denivelePuissances':
                                                                                                                {
                                                                                                                    'classesTemporelles': {
                                                                                                                        'classeTemporelle': [
                                                                                                                            {
                                                                                                                                'libelle': 'string',
                                                                                                                                'puissance': {
                                                                                                                                    'valeur': 1000.0,
                                                                                                                                    'unite': 'kVAR'},
                                                                                                                                'code': 'string'}]}},
                                                                                                            'calendrierFrn': {
                                                                                                                'libelle': 'string',
                                                                                                                'code': 'string'},
                                                                                                            'groupePeriodeMobile': {
                                                                                                                'libelle': 'string',
                                                                                                                'code': 'string'}}},
                'affaireId':
                    'string'}]}, 'interventionsEnCours': {'interventionEnCours': [{'affaireId': [
                'string'], 'teleoperable': False, 'operations': {'operation': [{'libelle':
                                                                                    'string', 'code': 'string'}],
                                                                 'referencesDisco': {'objet': {'libelle': 'string',
                                                                                               'code': 'string'},
                                                                                     'nature': {'libelle': 'string',
                                                                                                'code': 'string'}}},
                'dateCreationAc': '2012-05-17', 'datePrevue': '2001-06-26', 'heureDebut': '07:28:19',
                'heureFin': '13:49:34', 'creneauHorairePrevu': {'libelle': 'string', 'code': 'string'}}]},
            'historiqueAffaires': {'affaire': [{'prestation': {'fiche': {'libelle': 'string',
                                                                         'code': 'string'},
                                                               'option': {'libelle': 'string', 'code': 'string'},
                                                               'cas': {'libelle': 'string', 'code': 'string'},
                                                               'rang': 100}, 'refFournisseur':
                                                    'string', 'client': 'string',
                                                'nomCommercialActeurMarcheInitiateur': 'string',
                                                'dateEnvoi': '2005-03-29', 'dateEffet': '2003-06-10',
                                                'loginInitiateur': 'string',
                                                'etat': {'libelle': 'string', 'code': 'string'},
                                                'motifCloture': {'libelle': 'string', 'code': 'string'},
                                                'statut': {'libelle':
                                                               'string', 'code': 'ANNUL'}, 'id': 'string'}]},
            'elementsProchainesMesures': {
                'prochainCalculConsommation': {'nature': {'libelle': 'string', 'code': 'string'},
                                               'dateTheoriqueProchainCalculConsommation': '2015-12-31'},
                'dateTheoriqueProchainReleve': '2013-09-16'},
            'derniersIndexPublies': {'date': '2013-10-24', 'declencheur': {
                'libelle': 'string', 'code': 'string'}, 'nature': {'libelle': 'string',
                                                                   'code': 'string'},
                                     'origine': {'libelle': 'string', 'code': 'string'},
                                     'grilleTurpe': {'calendrier': {'libelle': 'string', 'code': 'string'},
                                                     'classesTemporelles': {
                                                         'classeTemporelle': [{'libelle': 'string', 'index': [{
                                                             'valeur': 1000.0,
                                                             'unite': 'string', 'grandeurPhysique':
                                                                 {'libelle': 'string', 'code': 'string'},
                                                             'phase': 100}], 'code': 'string'}]}},
                                     'grilleFrn': {'calendrier': {'libelle': 'string', 'code': 'string'},
                                                   'classesTemporelles': {
                                                       'classeTemporelle': [{'libelle': 'string', 'index': [{
                                                           'valeur': 1000.0, 'unite': 'string',
                                                           'grandeurPhysique':
                                                               {'libelle': 'string', 'code': 'string'}, 'phase': 100}],
                                                                             'code': 'string'}]}}},
            'derniersIndexReleves': {'date': '2000-09-30', 'declencheur': {
                'libelle': 'string', 'code': 'string'}, 'nature': {'libelle': 'string',
                                                                   'code': 'string'},
                                     'origine': {'libelle': 'string', 'code': 'string'},
                                     'grilleTurpe': {'calendrier': {'libelle': 'string', 'code': 'string'},
                                                     'classesTemporelles': {
                                                         'classeTemporelle': [{'libelle': 'string', 'index': [{
                                                             'valeur': 1000.0,
                                                             'unite': 'string', 'grandeurPhysique':
                                                                 {'libelle': 'string', 'code': 'string'},
                                                             'phase': 100}], 'code': 'string'}]}},
                                     'grilleFrn': {'calendrier': {'libelle': 'string', 'code': 'string'},
                                                   'classesTemporelles': {
                                                       'classeTemporelle': [{'libelle': 'string', 'index': [{
                                                           'valeur': 1000.0, 'unite': 'string',
                                                           'grandeurPhysique':
                                                               {'libelle': 'string', 'code': 'string'}, 'phase': 100}],
                                                                             'code': 'string'}]}}},
            'id': 'string'}}}

        self.assertDictEqual(reponse.json(), resultat_attendu)



from django.test import TestCase
from rest_framework.test import APIClient


class TestHealth(TestCase):

    def test_health(self):
        client = APIClient()
        reponse = client.get('/health/', format='json')
        body = reponse.json()
        self.assertEqual(body, 'ok')





from django.test import TestCase
from rest_framework.test import APIClient

from enedis.data_models.commanderarretservicesouscritmesures import DonneesGeneralesArretService, ArretServiceSouscrit, \
    DemandeArretServiceSouscritMesures


class TestViewCommanderArretServiceSouscrit(TestCase):
    maxDiff = None

    def test_view_commanderarretservicessouscrits(self):
        client = APIClient()
        arret_service_souscrit = ArretServiceSouscrit('12')
        donnees_generales = DonneesGeneralesArretService('12345678911234')
        demande_arret = DemandeArretServiceSouscritMesures(donnees_generales, arret_service_souscrit)
        entrees = {'serveur': 'poste_travail', 'donnees': demande_arret}
        reponse = client.post('/ws/v1.0/commanderarretservicessouscritmesures/', entrees, format='json')
        resultat_attendu = {'header': {
            'acquittement': None
        },
            'body': {
                'affaireId': 'string',
                'prestations': {
                    'prestation': [
                        {
                            'rang': 100,
                            'fiche': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'option': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'cas': {
                                'libelle': 'string', 'code': 'string'
                            }
                        }
                    ]
                }
            }
        }
        self.assertDictEqual(reponse.json(), resultat_attendu)


from enedis.connection.enedis_connection import EnedisConnection
from enedis.data_models.M023 import DemandeHistoriqueMesuresFines
from enedis.data_models.const import CadreAcces, SensMesure, TypeDonneesM023, FormatsM023
from enedis.services.commande_M023 import M023HistoriqueMesuresFines
import datetime as dt

def send_m023_demand():
    enedis_url = "ENEDIS_URL"
    login = "login"
    contract_id = "contract_id"
    path_public_cert = "path_public_cert"
    path_private_cert = "path_private_cert"

    # Récupération des variables d'env
    connection = EnedisConnection(enedis_url, login, contract_id, path_public_cert, path_private_cert)

    prms = ['00000000000001', '00000000000002']
    message = DemandeHistoriqueMesuresFines(dt.date(2024, 1, 1), dt.date(2024, 1, 31),
                                            CadreAcces.ACCORD_CLIENT,
                                            SensMesure.SOUTIRAGE,
                                            TypeDonneesM023.CDC,
                                            prms,
                                            FormatsM023.JSON)

    msg_sent, response = M023HistoriqueMesuresFines.m023(connection, message)



from pprint import pprint
from unittest.mock import patch

import zeep
from zeep.helpers import serialize_object

from enedis.data_models.commandercollectepublicationmesures import *
from enedis.data_models.common import PersonneMorale
from enedis.services.commander_collecte_publication_mesures import CommanderCollectePublicationMesures
from proxy.tests.services.ServicesTestCase import ServicesTestCase, raise_transport_errror, raise_creation_error


class TestCommaderCollectePublicationMesures(ServicesTestCase):


    def test_creation_message_correct(self):
        personne_morale = PersonneMorale('personnemoraleclient')
        declaration_accord_client = DeclarationAccordClient(personne_morale, accord=True)
        acces_mesures = AccesMesures(dt.date.today(), dt.date.today(), declaration_accord_client, TypeDonneesV1.INDEX, False,
                                     True, PasMesures.QUOTIDIEN, True,
                                     PeriodiciteTransmission.QUOTIDIENNE, False)
        donnees_generales = DonneesGeneralesDemandeCollecte('12345678911234')
        donnees = DemandeCollectePublicationMesure(donnees_generales, acces_mesures)

        msg, _ = CommanderCollectePublicationMesures.commander_collecte_publication(self.connection, donnees, send=False)
        expected_msg = {'demande': {
            'donneesGenerales': {
                'initiateurLogin': 'test@test.fr', 'contratId': '1111111', 'objetCode': 'AME', 'pointId': '12345678911234'
            },
            'accesMesures': {
                'dateDebut': dt.date.today(), 'dateFin': dt.date.today(), 'mesuresTypeCode': 'IDX',
                'mesuresPas': 'P1D', 'mesuresCorrigees': False, 'transmissionRecurrente': True,
                'periodiciteTransmission': 'P1D', 'injection': True, 'soutirage': False, 'declarationAccordClient': {
                    'accord': True,
                    'personneMorale': {
                        'denominationSociale': 'personnemoraleclient'}}}}}

        self.assertDictEqual(msg, expected_msg)

    def test_creation_message_malforme(self):
        personne_morale = PersonneMorale('personnemoraleclient')
        declaration_accord_client = DeclarationAccordClient(personne_morale, accord=True)
        acces_mesures = AccesMesures(None, dt.date.today(), declaration_accord_client, TypeDonneesV1.INDEX, False,
                                     True, PasMesures.QUOTIDIEN, True,
                                     PeriodiciteTransmission.QUOTIDIENNE, False)
        donnees_generales = DonneesGeneralesDemandeCollecte('12345678911234')
        donnees = DemandeCollectePublicationMesure(donnees_generales, acces_mesures)

        msg, response = CommanderCollectePublicationMesures.commander_collecte_publication(self.connection, donnees, send=True)
        self.assertEqual(response['erreur'], "('Missing element dateDebut',)", "le programme ne renvoie pas d'erreur alors qu'il manque un élément")

    def test_connexion(self):
        personne_morale = PersonneMorale('personnemoraleclient')
        declaration_accord_client = DeclarationAccordClient(personne_morale, accord=True)
        acces_mesures = AccesMesures(dt.date.today(), dt.date.today(), declaration_accord_client, TypeDonneesV1.INDEX,
                                     False,
                                     True, PasMesures.QUOTIDIEN, True,
                                     PeriodiciteTransmission.QUOTIDIENNE, False)
        donnees_generales = DonneesGeneralesDemandeCollecte('12345678911234')
        donnees = DemandeCollectePublicationMesure(donnees_generales, acces_mesures)
        _, response = CommanderCollectePublicationMesures.commander_collecte_publication(self.connection, donnees)

        expected_response = {'header': {
            'acquittement': None
        },
            'body': {
                'affaireId': 'string',
                'prestations': {
                    'prestation': [
                        {
                            'rang': 100,
                            'fiche': {
                                'libelle': 'string', 'code': 'string'
                            },
                            'option': {
                                'libelle': 'string', 'code': 'string'
                            }
                        }
                    ]
                }
                , 'serviceSouscritId': 'string'
            }
        }

        self.assertDictEqual(response, expected_response)

    @patch.object(zeep.helpers, 'serialize_object', raise_transport_errror)
    def test_connexion_echoue(self):
        personne_morale = PersonneMorale('personnemoraleclient')
        declaration_accord_client = DeclarationAccordClient(personne_morale, accord=True)
        acces_mesures = AccesMesures(dt.date.today(), dt.date.today(), declaration_accord_client, TypeDonneesV1.INDEX, False,
                                     True, PasMesures.QUOTIDIEN, True,
                                     PeriodiciteTransmission.QUOTIDIENNE, False)
        donnees_generales = DonneesGeneralesDemandeCollecte('12345678911234')
        donnees = DemandeCollectePublicationMesure(donnees_generales, acces_mesures)
        msg, response = CommanderCollectePublicationMesures.commander_collecte_publication(self.connection, donnees)
        self.assertEqual(response, {'erreur': "('une erreur inconnue est survenue',)", 'status': 'failed'})





from enedis.utils.regex import verification_prm


class DonneesConsulterPoint(dict):
    def __init__(self, point_id, autorisation_client=True, donnees_generales=True, modifications=True, interventions=True,
                 historique=True, elements_prochaines_mesures=True, derniers_index=True):

        verification_prm(point_id)

        donnees_demandees = {
            'donneesGeneralesPoint': donnees_generales,
            'modificationsContractuellesEnCours': modifications,
            'interventionsEnCours': interventions,
            'historiqueAffaires': historique,
            'elementsProchainesMesures': elements_prochaines_mesures,
            'derniersIndex': derniers_index
        }
        _kwargs = {'pointId': point_id,
                   'loginUtilisateur': '',
                   'donneesDemandees': donnees_demandees,
                   'autorisationClient': autorisation_client}

        dict.__init__(self, **_kwargs)


#   <xs:complexType name="ConsulterDonneesPointType">
#     <xs:sequence>
#       <xs:element name="pointId" type="ds:PointIdType"/>
#       <xs:element name="loginUtilisateur" type="ds:UtilisateurLoginType"/>
#       <xs:element name="donneesDemandees" type="sc:DonneesDemandeesType"/>
#       <xs:choice minOccurs="0">
#         <xs:element name="contratConcluNouveauClientSurSite" type="ds:BooleenType"/>
#         <xs:element name="autorisationClient" type="ds:BooleenType"/>
#       </xs:choice>
#     </xs:sequence>
#   </xs:complexType>

#   <xs:complexType name="DonneesDemandeesType">
#     <xs:sequence>
#       <xs:element name="donneesGeneralesPoint" type="ds:BooleenType"/>
#       <xs:element name="modificationsContractuellesEnCours" type="ds:BooleenType"/>
#       <xs:element name="interventionsEnCours" type="ds:BooleenType"/>
#       <xs:element name="historiqueAffaires" type="ds:BooleenType"/>
#       <xs:element name="elementsProchainesMesures" type="ds:BooleenType"/>
#       <xs:element name="derniersIndex" type="ds:BooleenType"/>
#     </xs:sequence>
#   </xs:complexType>